# Copyright 2023 The KServe Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# coding: utf-8

"""
    KServe

    Python SDK for KServe  # noqa: E501

    The version of the OpenAPI document: v0.1
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from kserve.configuration import Configuration


class V1alpha1InferenceGraphStatus(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'annotations': 'dict(str, str)',
        'conditions': 'list[KnativeCondition]',
        'observed_generation': 'int',
        'url': 'KnativeURL'
    }

    attribute_map = {
        'annotations': 'annotations',
        'conditions': 'conditions',
        'observed_generation': 'observedGeneration',
        'url': 'url'
    }

    def __init__(self, annotations=None, conditions=None, observed_generation=None, url=None, local_vars_configuration=None):  # noqa: E501
        """V1alpha1InferenceGraphStatus - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._annotations = None
        self._conditions = None
        self._observed_generation = None
        self._url = None
        self.discriminator = None

        if annotations is not None:
            self.annotations = annotations
        if conditions is not None:
            self.conditions = conditions
        if observed_generation is not None:
            self.observed_generation = observed_generation
        if url is not None:
            self.url = url

    @property
    def annotations(self):
        """Gets the annotations of this V1alpha1InferenceGraphStatus.  # noqa: E501

        Annotations is additional Status fields for the Resource to save some additional State as well as convey more information to the user. This is roughly akin to Annotations on any k8s resource, just the reconciler conveying richer information outwards.  # noqa: E501

        :return: The annotations of this V1alpha1InferenceGraphStatus.  # noqa: E501
        :rtype: dict(str, str)
        """
        return self._annotations

    @annotations.setter
    def annotations(self, annotations):
        """Sets the annotations of this V1alpha1InferenceGraphStatus.

        Annotations is additional Status fields for the Resource to save some additional State as well as convey more information to the user. This is roughly akin to Annotations on any k8s resource, just the reconciler conveying richer information outwards.  # noqa: E501

        :param annotations: The annotations of this V1alpha1InferenceGraphStatus.  # noqa: E501
        :type: dict(str, str)
        """

        self._annotations = annotations

    @property
    def conditions(self):
        """Gets the conditions of this V1alpha1InferenceGraphStatus.  # noqa: E501

        Conditions the latest available observations of a resource's current state.  # noqa: E501

        :return: The conditions of this V1alpha1InferenceGraphStatus.  # noqa: E501
        :rtype: list[KnativeCondition]
        """
        return self._conditions

    @conditions.setter
    def conditions(self, conditions):
        """Sets the conditions of this V1alpha1InferenceGraphStatus.

        Conditions the latest available observations of a resource's current state.  # noqa: E501

        :param conditions: The conditions of this V1alpha1InferenceGraphStatus.  # noqa: E501
        :type: list[KnativeCondition]
        """

        self._conditions = conditions

    @property
    def observed_generation(self):
        """Gets the observed_generation of this V1alpha1InferenceGraphStatus.  # noqa: E501

        ObservedGeneration is the 'Generation' of the Service that was last processed by the controller.  # noqa: E501

        :return: The observed_generation of this V1alpha1InferenceGraphStatus.  # noqa: E501
        :rtype: int
        """
        return self._observed_generation

    @observed_generation.setter
    def observed_generation(self, observed_generation):
        """Sets the observed_generation of this V1alpha1InferenceGraphStatus.

        ObservedGeneration is the 'Generation' of the Service that was last processed by the controller.  # noqa: E501

        :param observed_generation: The observed_generation of this V1alpha1InferenceGraphStatus.  # noqa: E501
        :type: int
        """

        self._observed_generation = observed_generation

    @property
    def url(self):
        """Gets the url of this V1alpha1InferenceGraphStatus.  # noqa: E501


        :return: The url of this V1alpha1InferenceGraphStatus.  # noqa: E501
        :rtype: KnativeURL
        """
        return self._url

    @url.setter
    def url(self, url):
        """Sets the url of this V1alpha1InferenceGraphStatus.


        :param url: The url of this V1alpha1InferenceGraphStatus.  # noqa: E501
        :type: KnativeURL
        """

        self._url = url

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, V1alpha1InferenceGraphStatus):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, V1alpha1InferenceGraphStatus):
            return True

        return self.to_dict() != other.to_dict()



# Copyright 2023 The KServe Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from typing import AsyncIterator, Union, List

from fastapi import Response
from starlette.datastructures import Headers

from kserve.protocol.rest.openai.types.openapi import CreateChatCompletionRequest
from kserve.protocol.rest.openai.types.openapi import (
    CreateChatCompletionResponse as ChatCompletion,
)
from kserve.protocol.rest.openai.types.openapi import (
    CreateChatCompletionStreamResponse as ChatCompletionChunk,
)
from kserve.protocol.rest.openai.types.openapi import CreateCompletionRequest
from kserve.protocol.rest.openai.types.openapi import (
    CreateCompletionResponse as Completion,
)
from kserve.protocol.rest.openai.types.openapi import CreateEmbeddingRequest
from kserve.protocol.rest.openai.types.openapi import (
    CreateEmbeddingResponse as Embedding,
)

from ...dataplane import DataPlane
from .openai_model import (
    ChatCompletionRequest,
    CompletionRequest,
    EmbeddingRequest,
    OpenAIModel,
    OpenAICompletionModel,
    OpenAIEmbeddingModel,
)


class OpenAIDataPlane(DataPlane):
    """OpenAI DataPlane"""

    async def create_completion(
        self,
        model_name: str,
        request: CreateCompletionRequest,
        headers: Headers,
        response: Response,
    ) -> Union[Completion, AsyncIterator[Completion]]:
        """Generate the text with the provided text prompt.

        Args:
            model_name (str): Model name.
            request (CreateCompletionRequest): Params to create a completion.
            headers: (Headers): Request headers.
            response: (Response): FastAPI response object
        Returns:
            response: A non-streaming or streaming completion response.

        Raises:
            InvalidInput: An error when the body bytes can't be decoded as JSON.
        """
        model = await self.get_model(model_name)
        if not isinstance(model, OpenAICompletionModel):
            raise RuntimeError(f"Model {model_name} does not support completion")

        completion_request = CompletionRequest(
            request_id=headers.get("x-request-id", None),
            params=request,
            context={"headers": dict(headers), "response": response},
        )
        return await model.create_completion(completion_request)

    async def create_chat_completion(
        self,
        model_name: str,
        request: CreateChatCompletionRequest,
        headers: Headers,
        response: Response,
    ) -> Union[ChatCompletion, AsyncIterator[ChatCompletionChunk]]:
        """Generate the text with the provided text prompt.

        Args:
            model_name (str): Model name.
            request (CreateChatCompletionRequest): Params to create a chat completion.
            headers: (Optional[Dict[str, str]]): Request headers.

        Returns:
            response: A non-streaming or streaming chat completion response

        Raises:
            InvalidInput: An error when the body bytes can't be decoded as JSON.
        """
        model = await self.get_model(model_name)
        if not isinstance(model, OpenAICompletionModel):
            raise RuntimeError(f"Model {model_name} does not support chat completion")

        completion_request = ChatCompletionRequest(
            request_id=headers.get("x-request-id", None),
            params=request,
            # We pass the response object in the context so it can be used to set response headers or a custom status code
            context={"headers": dict(headers), "response": response},
        )
        return await model.create_chat_completion(completion_request)

    async def create_embedding(
        self,
        model_name: str,
        request: CreateEmbeddingRequest,
        headers: Headers,
        response: Response,
    ) -> Embedding:
        """Creates an embedding vector representing the input text.

        Args:
            model_name (str): Model name.
            request (CreateEmbeddingRequest): Params to create the embedding.
            headers: (Optional[Dict[str, str]]): Request headers.

        Returns:
            response: A non-streaming embedding response

        Raises:
            InvalidInput: An error when the body bytes can't be decoded as JSON.
        """
        model = await self.get_model(model_name)
        if not isinstance(model, OpenAIEmbeddingModel):
            raise RuntimeError(f"Model {model_name} does not support embeddings")

        embedding_request = EmbeddingRequest(
            request_id=headers.get("x-request-id", None),
            params=request,
            context={"headers": dict(headers), "response": response},
        )
        return await model.create_embedding(embedding_request)

    async def models(self) -> List[OpenAIModel]:
        """Retrieve a list of models

        Returns:
            response: A list of OpenAIModel instances
        """
        return [
            model
            for model in self.model_registry.get_models().values()
            if isinstance(model, OpenAIModel)
        ]



# Copyright 2023 The KServe Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# coding: utf-8

"""
    KServe

    Python SDK for KServe  # noqa: E501

    The version of the OpenAPI document: v0.1
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from kserve.configuration import Configuration


class V1beta1ExplainerSpec(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'active_deadline_seconds': 'int',
        'affinity': 'V1Affinity',
        'annotations': 'dict(str, str)',
        'art': 'V1beta1ARTExplainerSpec',
        'automount_service_account_token': 'bool',
        'batcher': 'V1beta1Batcher',
        'canary_traffic_percent': 'int',
        'container_concurrency': 'int',
        'containers': 'list[V1Container]',
        'deployment_strategy': 'K8sIoApiAppsV1DeploymentStrategy',
        'dns_config': 'V1PodDNSConfig',
        'dns_policy': 'str',
        'enable_service_links': 'bool',
        'ephemeral_containers': 'list[V1EphemeralContainer]',
        'host_aliases': 'list[V1HostAlias]',
        'host_ipc': 'bool',
        'host_network': 'bool',
        'host_pid': 'bool',
        'host_users': 'bool',
        'hostname': 'str',
        'image_pull_secrets': 'list[V1LocalObjectReference]',
        'init_containers': 'list[V1Container]',
        'labels': 'dict(str, str)',
        'logger': 'V1beta1LoggerSpec',
        'max_replicas': 'int',
        'min_replicas': 'int',
        'node_name': 'str',
        'node_selector': 'dict(str, str)',
        'os': 'V1PodOS',
        'overhead': 'dict(str, ResourceQuantity)',
        'preemption_policy': 'str',
        'priority': 'int',
        'priority_class_name': 'str',
        'readiness_gates': 'list[V1PodReadinessGate]',
        'resource_claims': 'list[V1PodResourceClaim]',
        'restart_policy': 'str',
        'runtime_class_name': 'str',
        'scale_metric': 'str',
        'scale_target': 'int',
        'scheduler_name': 'str',
        'scheduling_gates': 'list[V1PodSchedulingGate]',
        'security_context': 'V1PodSecurityContext',
        'service_account': 'str',
        'service_account_name': 'str',
        'set_hostname_as_fqdn': 'bool',
        'share_process_namespace': 'bool',
        'subdomain': 'str',
        'termination_grace_period_seconds': 'int',
        'timeout': 'int',
        'tolerations': 'list[V1Toleration]',
        'topology_spread_constraints': 'list[V1TopologySpreadConstraint]',
        'volumes': 'list[V1Volume]'
    }

    attribute_map = {
        'active_deadline_seconds': 'activeDeadlineSeconds',
        'affinity': 'affinity',
        'annotations': 'annotations',
        'art': 'art',
        'automount_service_account_token': 'automountServiceAccountToken',
        'batcher': 'batcher',
        'canary_traffic_percent': 'canaryTrafficPercent',
        'container_concurrency': 'containerConcurrency',
        'containers': 'containers',
        'deployment_strategy': 'deploymentStrategy',
        'dns_config': 'dnsConfig',
        'dns_policy': 'dnsPolicy',
        'enable_service_links': 'enableServiceLinks',
        'ephemeral_containers': 'ephemeralContainers',
        'host_aliases': 'hostAliases',
        'host_ipc': 'hostIPC',
        'host_network': 'hostNetwork',
        'host_pid': 'hostPID',
        'host_users': 'hostUsers',
        'hostname': 'hostname',
        'image_pull_secrets': 'imagePullSecrets',
        'init_containers': 'initContainers',
        'labels': 'labels',
        'logger': 'logger',
        'max_replicas': 'maxReplicas',
        'min_replicas': 'minReplicas',
        'node_name': 'nodeName',
        'node_selector': 'nodeSelector',
        'os': 'os',
        'overhead': 'overhead',
        'preemption_policy': 'preemptionPolicy',
        'priority': 'priority',
        'priority_class_name': 'priorityClassName',
        'readiness_gates': 'readinessGates',
        'resource_claims': 'resourceClaims',
        'restart_policy': 'restartPolicy',
        'runtime_class_name': 'runtimeClassName',
        'scale_metric': 'scaleMetric',
        'scale_target': 'scaleTarget',
        'scheduler_name': 'schedulerName',
        'scheduling_gates': 'schedulingGates',
        'security_context': 'securityContext',
        'service_account': 'serviceAccount',
        'service_account_name': 'serviceAccountName',
        'set_hostname_as_fqdn': 'setHostnameAsFQDN',
        'share_process_namespace': 'shareProcessNamespace',
        'subdomain': 'subdomain',
        'termination_grace_period_seconds': 'terminationGracePeriodSeconds',
        'timeout': 'timeout',
        'tolerations': 'tolerations',
        'topology_spread_constraints': 'topologySpreadConstraints',
        'volumes': 'volumes'
    }

    def __init__(self, active_deadline_seconds=None, affinity=None, annotations=None, art=None, automount_service_account_token=None, batcher=None, canary_traffic_percent=None, container_concurrency=None, containers=None, deployment_strategy=None, dns_config=None, dns_policy=None, enable_service_links=None, ephemeral_containers=None, host_aliases=None, host_ipc=None, host_network=None, host_pid=None, host_users=None, hostname=None, image_pull_secrets=None, init_containers=None, labels=None, logger=None, max_replicas=None, min_replicas=None, node_name=None, node_selector=None, os=None, overhead=None, preemption_policy=None, priority=None, priority_class_name=None, readiness_gates=None, resource_claims=None, restart_policy=None, runtime_class_name=None, scale_metric=None, scale_target=None, scheduler_name=None, scheduling_gates=None, security_context=None, service_account=None, service_account_name=None, set_hostname_as_fqdn=None, share_process_namespace=None, subdomain=None, termination_grace_period_seconds=None, timeout=None, tolerations=None, topology_spread_constraints=None, volumes=None, local_vars_configuration=None):  # noqa: E501
        """V1beta1ExplainerSpec - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._active_deadline_seconds = None
        self._affinity = None
        self._annotations = None
        self._art = None
        self._automount_service_account_token = None
        self._batcher = None
        self._canary_traffic_percent = None
        self._container_concurrency = None
        self._containers = None
        self._deployment_strategy = None
        self._dns_config = None
        self._dns_policy = None
        self._enable_service_links = None
        self._ephemeral_containers = None
        self._host_aliases = None
        self._host_ipc = None
        self._host_network = None
        self._host_pid = None
        self._host_users = None
        self._hostname = None
        self._image_pull_secrets = None
        self._init_containers = None
        self._labels = None
        self._logger = None
        self._max_replicas = None
        self._min_replicas = None
        self._node_name = None
        self._node_selector = None
        self._os = None
        self._overhead = None
        self._preemption_policy = None
        self._priority = None
        self._priority_class_name = None
        self._readiness_gates = None
        self._resource_claims = None
        self._restart_policy = None
        self._runtime_class_name = None
        self._scale_metric = None
        self._scale_target = None
        self._scheduler_name = None
        self._scheduling_gates = None
        self._security_context = None
        self._service_account = None
        self._service_account_name = None
        self._set_hostname_as_fqdn = None
        self._share_process_namespace = None
        self._subdomain = None
        self._termination_grace_period_seconds = None
        self._timeout = None
        self._tolerations = None
        self._topology_spread_constraints = None
        self._volumes = None
        self.discriminator = None

        if active_deadline_seconds is not None:
            self.active_deadline_seconds = active_deadline_seconds
        if affinity is not None:
            self.affinity = affinity
        if annotations is not None:
            self.annotations = annotations
        if art is not None:
            self.art = art
        if automount_service_account_token is not None:
            self.automount_service_account_token = automount_service_account_token
        if batcher is not None:
            self.batcher = batcher
        if canary_traffic_percent is not None:
            self.canary_traffic_percent = canary_traffic_percent
        if container_concurrency is not None:
            self.container_concurrency = container_concurrency
        if containers is not None:
            self.containers = containers
        if deployment_strategy is not None:
            self.deployment_strategy = deployment_strategy
        if dns_config is not None:
            self.dns_config = dns_config
        if dns_policy is not None:
            self.dns_policy = dns_policy
        if enable_service_links is not None:
            self.enable_service_links = enable_service_links
        if ephemeral_containers is not None:
            self.ephemeral_containers = ephemeral_containers
        if host_aliases is not None:
            self.host_aliases = host_aliases
        if host_ipc is not None:
            self.host_ipc = host_ipc
        if host_network is not None:
            self.host_network = host_network
        if host_pid is not None:
            self.host_pid = host_pid
        if host_users is not None:
            self.host_users = host_users
        if hostname is not None:
            self.hostname = hostname
        if image_pull_secrets is not None:
            self.image_pull_secrets = image_pull_secrets
        if init_containers is not None:
            self.init_containers = init_containers
        if labels is not None:
            self.labels = labels
        if logger is not None:
            self.logger = logger
        if max_replicas is not None:
            self.max_replicas = max_replicas
        if min_replicas is not None:
            self.min_replicas = min_replicas
        if node_name is not None:
            self.node_name = node_name
        if node_selector is not None:
            self.node_selector = node_selector
        if os is not None:
            self.os = os
        if overhead is not None:
            self.overhead = overhead
        if preemption_policy is not None:
            self.preemption_policy = preemption_policy
        if priority is not None:
            self.priority = priority
        if priority_class_name is not None:
            self.priority_class_name = priority_class_name
        if readiness_gates is not None:
            self.readiness_gates = readiness_gates
        if resource_claims is not None:
            self.resource_claims = resource_claims
        if restart_policy is not None:
            self.restart_policy = restart_policy
        if runtime_class_name is not None:
            self.runtime_class_name = runtime_class_name
        if scale_metric is not None:
            self.scale_metric = scale_metric
        if scale_target is not None:
            self.scale_target = scale_target
        if scheduler_name is not None:
            self.scheduler_name = scheduler_name
        if scheduling_gates is not None:
            self.scheduling_gates = scheduling_gates
        if security_context is not None:
            self.security_context = security_context
        if service_account is not None:
            self.service_account = service_account
        if service_account_name is not None:
            self.service_account_name = service_account_name
        if set_hostname_as_fqdn is not None:
            self.set_hostname_as_fqdn = set_hostname_as_fqdn
        if share_process_namespace is not None:
            self.share_process_namespace = share_process_namespace
        if subdomain is not None:
            self.subdomain = subdomain
        if termination_grace_period_seconds is not None:
            self.termination_grace_period_seconds = termination_grace_period_seconds
        if timeout is not None:
            self.timeout = timeout
        if tolerations is not None:
            self.tolerations = tolerations
        if topology_spread_constraints is not None:
            self.topology_spread_constraints = topology_spread_constraints
        if volumes is not None:
            self.volumes = volumes

    @property
    def active_deadline_seconds(self):
        """Gets the active_deadline_seconds of this V1beta1ExplainerSpec.  # noqa: E501

        Optional duration in seconds the pod may be active on the node relative to StartTime before the system will actively try to mark it failed and kill associated containers. Value must be a positive integer.  # noqa: E501

        :return: The active_deadline_seconds of this V1beta1ExplainerSpec.  # noqa: E501
        :rtype: int
        """
        return self._active_deadline_seconds

    @active_deadline_seconds.setter
    def active_deadline_seconds(self, active_deadline_seconds):
        """Sets the active_deadline_seconds of this V1beta1ExplainerSpec.

        Optional duration in seconds the pod may be active on the node relative to StartTime before the system will actively try to mark it failed and kill associated containers. Value must be a positive integer.  # noqa: E501

        :param active_deadline_seconds: The active_deadline_seconds of this V1beta1ExplainerSpec.  # noqa: E501
        :type: int
        """

        self._active_deadline_seconds = active_deadline_seconds

    @property
    def affinity(self):
        """Gets the affinity of this V1beta1ExplainerSpec.  # noqa: E501


        :return: The affinity of this V1beta1ExplainerSpec.  # noqa: E501
        :rtype: V1Affinity
        """
        return self._affinity

    @affinity.setter
    def affinity(self, affinity):
        """Sets the affinity of this V1beta1ExplainerSpec.


        :param affinity: The affinity of this V1beta1ExplainerSpec.  # noqa: E501
        :type: V1Affinity
        """

        self._affinity = affinity

    @property
    def annotations(self):
        """Gets the annotations of this V1beta1ExplainerSpec.  # noqa: E501

        Annotations that will be added to the component pod. More info: https://kubernetes.io/docs/concepts/overview/working-with-objects/annotations/  # noqa: E501

        :return: The annotations of this V1beta1ExplainerSpec.  # noqa: E501
        :rtype: dict(str, str)
        """
        return self._annotations

    @annotations.setter
    def annotations(self, annotations):
        """Sets the annotations of this V1beta1ExplainerSpec.

        Annotations that will be added to the component pod. More info: https://kubernetes.io/docs/concepts/overview/working-with-objects/annotations/  # noqa: E501

        :param annotations: The annotations of this V1beta1ExplainerSpec.  # noqa: E501
        :type: dict(str, str)
        """

        self._annotations = annotations

    @property
    def art(self):
        """Gets the art of this V1beta1ExplainerSpec.  # noqa: E501


        :return: The art of this V1beta1ExplainerSpec.  # noqa: E501
        :rtype: V1beta1ARTExplainerSpec
        """
        return self._art

    @art.setter
    def art(self, art):
        """Sets the art of this V1beta1ExplainerSpec.


        :param art: The art of this V1beta1ExplainerSpec.  # noqa: E501
        :type: V1beta1ARTExplainerSpec
        """

        self._art = art

    @property
    def automount_service_account_token(self):
        """Gets the automount_service_account_token of this V1beta1ExplainerSpec.  # noqa: E501

        AutomountServiceAccountToken indicates whether a service account token should be automatically mounted.  # noqa: E501

        :return: The automount_service_account_token of this V1beta1ExplainerSpec.  # noqa: E501
        :rtype: bool
        """
        return self._automount_service_account_token

    @automount_service_account_token.setter
    def automount_service_account_token(self, automount_service_account_token):
        """Sets the automount_service_account_token of this V1beta1ExplainerSpec.

        AutomountServiceAccountToken indicates whether a service account token should be automatically mounted.  # noqa: E501

        :param automount_service_account_token: The automount_service_account_token of this V1beta1ExplainerSpec.  # noqa: E501
        :type: bool
        """

        self._automount_service_account_token = automount_service_account_token

    @property
    def batcher(self):
        """Gets the batcher of this V1beta1ExplainerSpec.  # noqa: E501


        :return: The batcher of this V1beta1ExplainerSpec.  # noqa: E501
        :rtype: V1beta1Batcher
        """
        return self._batcher

    @batcher.setter
    def batcher(self, batcher):
        """Sets the batcher of this V1beta1ExplainerSpec.


        :param batcher: The batcher of this V1beta1ExplainerSpec.  # noqa: E501
        :type: V1beta1Batcher
        """

        self._batcher = batcher

    @property
    def canary_traffic_percent(self):
        """Gets the canary_traffic_percent of this V1beta1ExplainerSpec.  # noqa: E501

        CanaryTrafficPercent defines the traffic split percentage between the candidate revision and the last ready revision  # noqa: E501

        :return: The canary_traffic_percent of this V1beta1ExplainerSpec.  # noqa: E501
        :rtype: int
        """
        return self._canary_traffic_percent

    @canary_traffic_percent.setter
    def canary_traffic_percent(self, canary_traffic_percent):
        """Sets the canary_traffic_percent of this V1beta1ExplainerSpec.

        CanaryTrafficPercent defines the traffic split percentage between the candidate revision and the last ready revision  # noqa: E501

        :param canary_traffic_percent: The canary_traffic_percent of this V1beta1ExplainerSpec.  # noqa: E501
        :type: int
        """

        self._canary_traffic_percent = canary_traffic_percent

    @property
    def container_concurrency(self):
        """Gets the container_concurrency of this V1beta1ExplainerSpec.  # noqa: E501

        ContainerConcurrency specifies how many requests can be processed concurrently, this sets the hard limit of the container concurrency(https://knative.dev/docs/serving/autoscaling/concurrency).  # noqa: E501

        :return: The container_concurrency of this V1beta1ExplainerSpec.  # noqa: E501
        :rtype: int
        """
        return self._container_concurrency

    @container_concurrency.setter
    def container_concurrency(self, container_concurrency):
        """Sets the container_concurrency of this V1beta1ExplainerSpec.

        ContainerConcurrency specifies how many requests can be processed concurrently, this sets the hard limit of the container concurrency(https://knative.dev/docs/serving/autoscaling/concurrency).  # noqa: E501

        :param container_concurrency: The container_concurrency of this V1beta1ExplainerSpec.  # noqa: E501
        :type: int
        """

        self._container_concurrency = container_concurrency

    @property
    def containers(self):
        """Gets the containers of this V1beta1ExplainerSpec.  # noqa: E501

        List of containers belonging to the pod. Containers cannot currently be added or removed. There must be at least one container in a Pod. Cannot be updated.  # noqa: E501

        :return: The containers of this V1beta1ExplainerSpec.  # noqa: E501
        :rtype: list[V1Container]
        """
        return self._containers

    @containers.setter
    def containers(self, containers):
        """Sets the containers of this V1beta1ExplainerSpec.

        List of containers belonging to the pod. Containers cannot currently be added or removed. There must be at least one container in a Pod. Cannot be updated.  # noqa: E501

        :param containers: The containers of this V1beta1ExplainerSpec.  # noqa: E501
        :type: list[V1Container]
        """

        self._containers = containers

    @property
    def deployment_strategy(self):
        """Gets the deployment_strategy of this V1beta1ExplainerSpec.  # noqa: E501


        :return: The deployment_strategy of this V1beta1ExplainerSpec.  # noqa: E501
        :rtype: K8sIoApiAppsV1DeploymentStrategy
        """
        return self._deployment_strategy

    @deployment_strategy.setter
    def deployment_strategy(self, deployment_strategy):
        """Sets the deployment_strategy of this V1beta1ExplainerSpec.


        :param deployment_strategy: The deployment_strategy of this V1beta1ExplainerSpec.  # noqa: E501
        :type: K8sIoApiAppsV1DeploymentStrategy
        """

        self._deployment_strategy = deployment_strategy

    @property
    def dns_config(self):
        """Gets the dns_config of this V1beta1ExplainerSpec.  # noqa: E501


        :return: The dns_config of this V1beta1ExplainerSpec.  # noqa: E501
        :rtype: V1PodDNSConfig
        """
        return self._dns_config

    @dns_config.setter
    def dns_config(self, dns_config):
        """Sets the dns_config of this V1beta1ExplainerSpec.


        :param dns_config: The dns_config of this V1beta1ExplainerSpec.  # noqa: E501
        :type: V1PodDNSConfig
        """

        self._dns_config = dns_config

    @property
    def dns_policy(self):
        """Gets the dns_policy of this V1beta1ExplainerSpec.  # noqa: E501

        Set DNS policy for the pod. Defaults to \"ClusterFirst\". Valid values are 'ClusterFirstWithHostNet', 'ClusterFirst', 'Default' or 'None'. DNS parameters given in DNSConfig will be merged with the policy selected with DNSPolicy. To have DNS options set along with hostNetwork, you have to specify DNS policy explicitly to 'ClusterFirstWithHostNet'.  # noqa: E501

        :return: The dns_policy of this V1beta1ExplainerSpec.  # noqa: E501
        :rtype: str
        """
        return self._dns_policy

    @dns_policy.setter
    def dns_policy(self, dns_policy):
        """Sets the dns_policy of this V1beta1ExplainerSpec.

        Set DNS policy for the pod. Defaults to \"ClusterFirst\". Valid values are 'ClusterFirstWithHostNet', 'ClusterFirst', 'Default' or 'None'. DNS parameters given in DNSConfig will be merged with the policy selected with DNSPolicy. To have DNS options set along with hostNetwork, you have to specify DNS policy explicitly to 'ClusterFirstWithHostNet'.  # noqa: E501

        :param dns_policy: The dns_policy of this V1beta1ExplainerSpec.  # noqa: E501
        :type: str
        """

        self._dns_policy = dns_policy

    @property
    def enable_service_links(self):
        """Gets the enable_service_links of this V1beta1ExplainerSpec.  # noqa: E501

        EnableServiceLinks indicates whether information about services should be injected into pod's environment variables, matching the syntax of Docker links. Optional: Defaults to true.  # noqa: E501

        :return: The enable_service_links of this V1beta1ExplainerSpec.  # noqa: E501
        :rtype: bool
        """
        return self._enable_service_links

    @enable_service_links.setter
    def enable_service_links(self, enable_service_links):
        """Sets the enable_service_links of this V1beta1ExplainerSpec.

        EnableServiceLinks indicates whether information about services should be injected into pod's environment variables, matching the syntax of Docker links. Optional: Defaults to true.  # noqa: E501

        :param enable_service_links: The enable_service_links of this V1beta1ExplainerSpec.  # noqa: E501
        :type: bool
        """

        self._enable_service_links = enable_service_links

    @property
    def ephemeral_containers(self):
        """Gets the ephemeral_containers of this V1beta1ExplainerSpec.  # noqa: E501

        List of ephemeral containers run in this pod. Ephemeral containers may be run in an existing pod to perform user-initiated actions such as debugging. This list cannot be specified when creating a pod, and it cannot be modified by updating the pod spec. In order to add an ephemeral container to an existing pod, use the pod's ephemeralcontainers subresource. This field is beta-level and available on clusters that haven't disabled the EphemeralContainers feature gate.  # noqa: E501

        :return: The ephemeral_containers of this V1beta1ExplainerSpec.  # noqa: E501
        :rtype: list[V1EphemeralContainer]
        """
        return self._ephemeral_containers

    @ephemeral_containers.setter
    def ephemeral_containers(self, ephemeral_containers):
        """Sets the ephemeral_containers of this V1beta1ExplainerSpec.

        List of ephemeral containers run in this pod. Ephemeral containers may be run in an existing pod to perform user-initiated actions such as debugging. This list cannot be specified when creating a pod, and it cannot be modified by updating the pod spec. In order to add an ephemeral container to an existing pod, use the pod's ephemeralcontainers subresource. This field is beta-level and available on clusters that haven't disabled the EphemeralContainers feature gate.  # noqa: E501

        :param ephemeral_containers: The ephemeral_containers of this V1beta1ExplainerSpec.  # noqa: E501
        :type: list[V1EphemeralContainer]
        """

        self._ephemeral_containers = ephemeral_containers

    @property
    def host_aliases(self):
        """Gets the host_aliases of this V1beta1ExplainerSpec.  # noqa: E501

        HostAliases is an optional list of hosts and IPs that will be injected into the pod's hosts file if specified. This is only valid for non-hostNetwork pods.  # noqa: E501

        :return: The host_aliases of this V1beta1ExplainerSpec.  # noqa: E501
        :rtype: list[V1HostAlias]
        """
        return self._host_aliases

    @host_aliases.setter
    def host_aliases(self, host_aliases):
        """Sets the host_aliases of this V1beta1ExplainerSpec.

        HostAliases is an optional list of hosts and IPs that will be injected into the pod's hosts file if specified. This is only valid for non-hostNetwork pods.  # noqa: E501

        :param host_aliases: The host_aliases of this V1beta1ExplainerSpec.  # noqa: E501
        :type: list[V1HostAlias]
        """

        self._host_aliases = host_aliases

    @property
    def host_ipc(self):
        """Gets the host_ipc of this V1beta1ExplainerSpec.  # noqa: E501

        Use the host's ipc namespace. Optional: Default to false.  # noqa: E501

        :return: The host_ipc of this V1beta1ExplainerSpec.  # noqa: E501
        :rtype: bool
        """
        return self._host_ipc

    @host_ipc.setter
    def host_ipc(self, host_ipc):
        """Sets the host_ipc of this V1beta1ExplainerSpec.

        Use the host's ipc namespace. Optional: Default to false.  # noqa: E501

        :param host_ipc: The host_ipc of this V1beta1ExplainerSpec.  # noqa: E501
        :type: bool
        """

        self._host_ipc = host_ipc

    @property
    def host_network(self):
        """Gets the host_network of this V1beta1ExplainerSpec.  # noqa: E501

        Host networking requested for this pod. Use the host's network namespace. If this option is set, the ports that will be used must be specified. Default to false.  # noqa: E501

        :return: The host_network of this V1beta1ExplainerSpec.  # noqa: E501
        :rtype: bool
        """
        return self._host_network

    @host_network.setter
    def host_network(self, host_network):
        """Sets the host_network of this V1beta1ExplainerSpec.

        Host networking requested for this pod. Use the host's network namespace. If this option is set, the ports that will be used must be specified. Default to false.  # noqa: E501

        :param host_network: The host_network of this V1beta1ExplainerSpec.  # noqa: E501
        :type: bool
        """

        self._host_network = host_network

    @property
    def host_pid(self):
        """Gets the host_pid of this V1beta1ExplainerSpec.  # noqa: E501

        Use the host's pid namespace. Optional: Default to false.  # noqa: E501

        :return: The host_pid of this V1beta1ExplainerSpec.  # noqa: E501
        :rtype: bool
        """
        return self._host_pid

    @host_pid.setter
    def host_pid(self, host_pid):
        """Sets the host_pid of this V1beta1ExplainerSpec.

        Use the host's pid namespace. Optional: Default to false.  # noqa: E501

        :param host_pid: The host_pid of this V1beta1ExplainerSpec.  # noqa: E501
        :type: bool
        """

        self._host_pid = host_pid

    @property
    def host_users(self):
        """Gets the host_users of this V1beta1ExplainerSpec.  # noqa: E501

        Use the host's user namespace. Optional: Default to true. If set to true or not present, the pod will be run in the host user namespace, useful for when the pod needs a feature only available to the host user namespace, such as loading a kernel module with CAP_SYS_MODULE. When set to false, a new userns is created for the pod. Setting false is useful for mitigating container breakout vulnerabilities even allowing users to run their containers as root without actually having root privileges on the host. This field is alpha-level and is only honored by servers that enable the UserNamespacesSupport feature.  # noqa: E501

        :return: The host_users of this V1beta1ExplainerSpec.  # noqa: E501
        :rtype: bool
        """
        return self._host_users

    @host_users.setter
    def host_users(self, host_users):
        """Sets the host_users of this V1beta1ExplainerSpec.

        Use the host's user namespace. Optional: Default to true. If set to true or not present, the pod will be run in the host user namespace, useful for when the pod needs a feature only available to the host user namespace, such as loading a kernel module with CAP_SYS_MODULE. When set to false, a new userns is created for the pod. Setting false is useful for mitigating container breakout vulnerabilities even allowing users to run their containers as root without actually having root privileges on the host. This field is alpha-level and is only honored by servers that enable the UserNamespacesSupport feature.  # noqa: E501

        :param host_users: The host_users of this V1beta1ExplainerSpec.  # noqa: E501
        :type: bool
        """

        self._host_users = host_users

    @property
    def hostname(self):
        """Gets the hostname of this V1beta1ExplainerSpec.  # noqa: E501

        Specifies the hostname of the Pod If not specified, the pod's hostname will be set to a system-defined value.  # noqa: E501

        :return: The hostname of this V1beta1ExplainerSpec.  # noqa: E501
        :rtype: str
        """
        return self._hostname

    @hostname.setter
    def hostname(self, hostname):
        """Sets the hostname of this V1beta1ExplainerSpec.

        Specifies the hostname of the Pod If not specified, the pod's hostname will be set to a system-defined value.  # noqa: E501

        :param hostname: The hostname of this V1beta1ExplainerSpec.  # noqa: E501
        :type: str
        """

        self._hostname = hostname

    @property
    def image_pull_secrets(self):
        """Gets the image_pull_secrets of this V1beta1ExplainerSpec.  # noqa: E501

        ImagePullSecrets is an optional list of references to secrets in the same namespace to use for pulling any of the images used by this PodSpec. If specified, these secrets will be passed to individual puller implementations for them to use. For example, in the case of docker, only DockerConfig type secrets are honored. More info: https://kubernetes.io/docs/concepts/containers/images#specifying-imagepullsecrets-on-a-pod  # noqa: E501

        :return: The image_pull_secrets of this V1beta1ExplainerSpec.  # noqa: E501
        :rtype: list[V1LocalObjectReference]
        """
        return self._image_pull_secrets

    @image_pull_secrets.setter
    def image_pull_secrets(self, image_pull_secrets):
        """Sets the image_pull_secrets of this V1beta1ExplainerSpec.

        ImagePullSecrets is an optional list of references to secrets in the same namespace to use for pulling any of the images used by this PodSpec. If specified, these secrets will be passed to individual puller implementations for them to use. For example, in the case of docker, only DockerConfig type secrets are honored. More info: https://kubernetes.io/docs/concepts/containers/images#specifying-imagepullsecrets-on-a-pod  # noqa: E501

        :param image_pull_secrets: The image_pull_secrets of this V1beta1ExplainerSpec.  # noqa: E501
        :type: list[V1LocalObjectReference]
        """

        self._image_pull_secrets = image_pull_secrets

    @property
    def init_containers(self):
        """Gets the init_containers of this V1beta1ExplainerSpec.  # noqa: E501

        List of initialization containers belonging to the pod. Init containers are executed in order prior to containers being started. If any init container fails, the pod is considered to have failed and is handled according to its restartPolicy. The name for an init container or normal container must be unique among all containers. Init containers may not have Lifecycle actions, Readiness probes, Liveness probes, or Startup probes. The resourceRequirements of an init container are taken into account during scheduling by finding the highest request/limit for each resource type, and then using the max of of that value or the sum of the normal containers. Limits are applied to init containers in a similar fashion. Init containers cannot currently be added or removed. Cannot be updated. More info: https://kubernetes.io/docs/concepts/workloads/pods/init-containers/  # noqa: E501

        :return: The init_containers of this V1beta1ExplainerSpec.  # noqa: E501
        :rtype: list[V1Container]
        """
        return self._init_containers

    @init_containers.setter
    def init_containers(self, init_containers):
        """Sets the init_containers of this V1beta1ExplainerSpec.

        List of initialization containers belonging to the pod. Init containers are executed in order prior to containers being started. If any init container fails, the pod is considered to have failed and is handled according to its restartPolicy. The name for an init container or normal container must be unique among all containers. Init containers may not have Lifecycle actions, Readiness probes, Liveness probes, or Startup probes. The resourceRequirements of an init container are taken into account during scheduling by finding the highest request/limit for each resource type, and then using the max of of that value or the sum of the normal containers. Limits are applied to init containers in a similar fashion. Init containers cannot currently be added or removed. Cannot be updated. More info: https://kubernetes.io/docs/concepts/workloads/pods/init-containers/  # noqa: E501

        :param init_containers: The init_containers of this V1beta1ExplainerSpec.  # noqa: E501
        :type: list[V1Container]
        """

        self._init_containers = init_containers

    @property
    def labels(self):
        """Gets the labels of this V1beta1ExplainerSpec.  # noqa: E501

        Labels that will be added to the component pod. More info: https://kubernetes.io/docs/concepts/overview/working-with-objects/labels/  # noqa: E501

        :return: The labels of this V1beta1ExplainerSpec.  # noqa: E501
        :rtype: dict(str, str)
        """
        return self._labels

    @labels.setter
    def labels(self, labels):
        """Sets the labels of this V1beta1ExplainerSpec.

        Labels that will be added to the component pod. More info: https://kubernetes.io/docs/concepts/overview/working-with-objects/labels/  # noqa: E501

        :param labels: The labels of this V1beta1ExplainerSpec.  # noqa: E501
        :type: dict(str, str)
        """

        self._labels = labels

    @property
    def logger(self):
        """Gets the logger of this V1beta1ExplainerSpec.  # noqa: E501


        :return: The logger of this V1beta1ExplainerSpec.  # noqa: E501
        :rtype: V1beta1LoggerSpec
        """
        return self._logger

    @logger.setter
    def logger(self, logger):
        """Sets the logger of this V1beta1ExplainerSpec.


        :param logger: The logger of this V1beta1ExplainerSpec.  # noqa: E501
        :type: V1beta1LoggerSpec
        """

        self._logger = logger

    @property
    def max_replicas(self):
        """Gets the max_replicas of this V1beta1ExplainerSpec.  # noqa: E501

        Maximum number of replicas for autoscaling.  # noqa: E501

        :return: The max_replicas of this V1beta1ExplainerSpec.  # noqa: E501
        :rtype: int
        """
        return self._max_replicas

    @max_replicas.setter
    def max_replicas(self, max_replicas):
        """Sets the max_replicas of this V1beta1ExplainerSpec.

        Maximum number of replicas for autoscaling.  # noqa: E501

        :param max_replicas: The max_replicas of this V1beta1ExplainerSpec.  # noqa: E501
        :type: int
        """

        self._max_replicas = max_replicas

    @property
    def min_replicas(self):
        """Gets the min_replicas of this V1beta1ExplainerSpec.  # noqa: E501

        Minimum number of replicas, defaults to 1 but can be set to 0 to enable scale-to-zero.  # noqa: E501

        :return: The min_replicas of this V1beta1ExplainerSpec.  # noqa: E501
        :rtype: int
        """
        return self._min_replicas

    @min_replicas.setter
    def min_replicas(self, min_replicas):
        """Sets the min_replicas of this V1beta1ExplainerSpec.

        Minimum number of replicas, defaults to 1 but can be set to 0 to enable scale-to-zero.  # noqa: E501

        :param min_replicas: The min_replicas of this V1beta1ExplainerSpec.  # noqa: E501
        :type: int
        """

        self._min_replicas = min_replicas

    @property
    def node_name(self):
        """Gets the node_name of this V1beta1ExplainerSpec.  # noqa: E501

        NodeName is a request to schedule this pod onto a specific node. If it is non-empty, the scheduler simply schedules this pod onto that node, assuming that it fits resource requirements.  # noqa: E501

        :return: The node_name of this V1beta1ExplainerSpec.  # noqa: E501
        :rtype: str
        """
        return self._node_name

    @node_name.setter
    def node_name(self, node_name):
        """Sets the node_name of this V1beta1ExplainerSpec.

        NodeName is a request to schedule this pod onto a specific node. If it is non-empty, the scheduler simply schedules this pod onto that node, assuming that it fits resource requirements.  # noqa: E501

        :param node_name: The node_name of this V1beta1ExplainerSpec.  # noqa: E501
        :type: str
        """

        self._node_name = node_name

    @property
    def node_selector(self):
        """Gets the node_selector of this V1beta1ExplainerSpec.  # noqa: E501

        NodeSelector is a selector which must be true for the pod to fit on a node. Selector which must match a node's labels for the pod to be scheduled on that node. More info: https://kubernetes.io/docs/concepts/configuration/assign-pod-node/  # noqa: E501

        :return: The node_selector of this V1beta1ExplainerSpec.  # noqa: E501
        :rtype: dict(str, str)
        """
        return self._node_selector

    @node_selector.setter
    def node_selector(self, node_selector):
        """Sets the node_selector of this V1beta1ExplainerSpec.

        NodeSelector is a selector which must be true for the pod to fit on a node. Selector which must match a node's labels for the pod to be scheduled on that node. More info: https://kubernetes.io/docs/concepts/configuration/assign-pod-node/  # noqa: E501

        :param node_selector: The node_selector of this V1beta1ExplainerSpec.  # noqa: E501
        :type: dict(str, str)
        """

        self._node_selector = node_selector

    @property
    def os(self):
        """Gets the os of this V1beta1ExplainerSpec.  # noqa: E501


        :return: The os of this V1beta1ExplainerSpec.  # noqa: E501
        :rtype: V1PodOS
        """
        return self._os

    @os.setter
    def os(self, os):
        """Sets the os of this V1beta1ExplainerSpec.


        :param os: The os of this V1beta1ExplainerSpec.  # noqa: E501
        :type: V1PodOS
        """

        self._os = os

    @property
    def overhead(self):
        """Gets the overhead of this V1beta1ExplainerSpec.  # noqa: E501

        Overhead represents the resource overhead associated with running a pod for a given RuntimeClass. This field will be autopopulated at admission time by the RuntimeClass admission controller. If the RuntimeClass admission controller is enabled, overhead must not be set in Pod create requests. The RuntimeClass admission controller will reject Pod create requests which have the overhead already set. If RuntimeClass is configured and selected in the PodSpec, Overhead will be set to the value defined in the corresponding RuntimeClass, otherwise it will remain unset and treated as zero. More info: https://git.k8s.io/enhancements/keps/sig-node/688-pod-overhead/README.md This field is beta-level as of Kubernetes v1.18, and is only honored by servers that enable the PodOverhead feature.  # noqa: E501

        :return: The overhead of this V1beta1ExplainerSpec.  # noqa: E501
        :rtype: dict(str, ResourceQuantity)
        """
        return self._overhead

    @overhead.setter
    def overhead(self, overhead):
        """Sets the overhead of this V1beta1ExplainerSpec.

        Overhead represents the resource overhead associated with running a pod for a given RuntimeClass. This field will be autopopulated at admission time by the RuntimeClass admission controller. If the RuntimeClass admission controller is enabled, overhead must not be set in Pod create requests. The RuntimeClass admission controller will reject Pod create requests which have the overhead already set. If RuntimeClass is configured and selected in the PodSpec, Overhead will be set to the value defined in the corresponding RuntimeClass, otherwise it will remain unset and treated as zero. More info: https://git.k8s.io/enhancements/keps/sig-node/688-pod-overhead/README.md This field is beta-level as of Kubernetes v1.18, and is only honored by servers that enable the PodOverhead feature.  # noqa: E501

        :param overhead: The overhead of this V1beta1ExplainerSpec.  # noqa: E501
        :type: dict(str, ResourceQuantity)
        """

        self._overhead = overhead

    @property
    def preemption_policy(self):
        """Gets the preemption_policy of this V1beta1ExplainerSpec.  # noqa: E501

        PreemptionPolicy is the Policy for preempting pods with lower priority. One of Never, PreemptLowerPriority. Defaults to PreemptLowerPriority if unset. This field is beta-level, gated by the NonPreemptingPriority feature-gate.  # noqa: E501

        :return: The preemption_policy of this V1beta1ExplainerSpec.  # noqa: E501
        :rtype: str
        """
        return self._preemption_policy

    @preemption_policy.setter
    def preemption_policy(self, preemption_policy):
        """Sets the preemption_policy of this V1beta1ExplainerSpec.

        PreemptionPolicy is the Policy for preempting pods with lower priority. One of Never, PreemptLowerPriority. Defaults to PreemptLowerPriority if unset. This field is beta-level, gated by the NonPreemptingPriority feature-gate.  # noqa: E501

        :param preemption_policy: The preemption_policy of this V1beta1ExplainerSpec.  # noqa: E501
        :type: str
        """

        self._preemption_policy = preemption_policy

    @property
    def priority(self):
        """Gets the priority of this V1beta1ExplainerSpec.  # noqa: E501

        The priority value. Various system components use this field to find the priority of the pod. When Priority Admission Controller is enabled, it prevents users from setting this field. The admission controller populates this field from PriorityClassName. The higher the value, the higher the priority.  # noqa: E501

        :return: The priority of this V1beta1ExplainerSpec.  # noqa: E501
        :rtype: int
        """
        return self._priority

    @priority.setter
    def priority(self, priority):
        """Sets the priority of this V1beta1ExplainerSpec.

        The priority value. Various system components use this field to find the priority of the pod. When Priority Admission Controller is enabled, it prevents users from setting this field. The admission controller populates this field from PriorityClassName. The higher the value, the higher the priority.  # noqa: E501

        :param priority: The priority of this V1beta1ExplainerSpec.  # noqa: E501
        :type: int
        """

        self._priority = priority

    @property
    def priority_class_name(self):
        """Gets the priority_class_name of this V1beta1ExplainerSpec.  # noqa: E501

        If specified, indicates the pod's priority. \"system-node-critical\" and \"system-cluster-critical\" are two special keywords which indicate the highest priorities with the former being the highest priority. Any other name must be defined by creating a PriorityClass object with that name. If not specified, the pod priority will be default or zero if there is no default.  # noqa: E501

        :return: The priority_class_name of this V1beta1ExplainerSpec.  # noqa: E501
        :rtype: str
        """
        return self._priority_class_name

    @priority_class_name.setter
    def priority_class_name(self, priority_class_name):
        """Sets the priority_class_name of this V1beta1ExplainerSpec.

        If specified, indicates the pod's priority. \"system-node-critical\" and \"system-cluster-critical\" are two special keywords which indicate the highest priorities with the former being the highest priority. Any other name must be defined by creating a PriorityClass object with that name. If not specified, the pod priority will be default or zero if there is no default.  # noqa: E501

        :param priority_class_name: The priority_class_name of this V1beta1ExplainerSpec.  # noqa: E501
        :type: str
        """

        self._priority_class_name = priority_class_name

    @property
    def readiness_gates(self):
        """Gets the readiness_gates of this V1beta1ExplainerSpec.  # noqa: E501

        If specified, all readiness gates will be evaluated for pod readiness. A pod is ready when all its containers are ready AND all conditions specified in the readiness gates have status equal to \"True\" More info: https://git.k8s.io/enhancements/keps/sig-network/580-pod-readiness-gates  # noqa: E501

        :return: The readiness_gates of this V1beta1ExplainerSpec.  # noqa: E501
        :rtype: list[V1PodReadinessGate]
        """
        return self._readiness_gates

    @readiness_gates.setter
    def readiness_gates(self, readiness_gates):
        """Sets the readiness_gates of this V1beta1ExplainerSpec.

        If specified, all readiness gates will be evaluated for pod readiness. A pod is ready when all its containers are ready AND all conditions specified in the readiness gates have status equal to \"True\" More info: https://git.k8s.io/enhancements/keps/sig-network/580-pod-readiness-gates  # noqa: E501

        :param readiness_gates: The readiness_gates of this V1beta1ExplainerSpec.  # noqa: E501
        :type: list[V1PodReadinessGate]
        """

        self._readiness_gates = readiness_gates

    @property
    def resource_claims(self):
        """Gets the resource_claims of this V1beta1ExplainerSpec.  # noqa: E501

        ResourceClaims defines which ResourceClaims must be allocated and reserved before the Pod is allowed to start. The resources will be made available to those containers which consume them by name.  This is an alpha field and requires enabling the DynamicResourceAllocation feature gate.  This field is immutable.  # noqa: E501

        :return: The resource_claims of this V1beta1ExplainerSpec.  # noqa: E501
        :rtype: list[V1PodResourceClaim]
        """
        return self._resource_claims

    @resource_claims.setter
    def resource_claims(self, resource_claims):
        """Sets the resource_claims of this V1beta1ExplainerSpec.

        ResourceClaims defines which ResourceClaims must be allocated and reserved before the Pod is allowed to start. The resources will be made available to those containers which consume them by name.  This is an alpha field and requires enabling the DynamicResourceAllocation feature gate.  This field is immutable.  # noqa: E501

        :param resource_claims: The resource_claims of this V1beta1ExplainerSpec.  # noqa: E501
        :type: list[V1PodResourceClaim]
        """

        self._resource_claims = resource_claims

    @property
    def restart_policy(self):
        """Gets the restart_policy of this V1beta1ExplainerSpec.  # noqa: E501

        Restart policy for all containers within the pod. One of Always, OnFailure, Never. Default to Always. More info: https://kubernetes.io/docs/concepts/workloads/pods/pod-lifecycle/#restart-policy  # noqa: E501

        :return: The restart_policy of this V1beta1ExplainerSpec.  # noqa: E501
        :rtype: str
        """
        return self._restart_policy

    @restart_policy.setter
    def restart_policy(self, restart_policy):
        """Sets the restart_policy of this V1beta1ExplainerSpec.

        Restart policy for all containers within the pod. One of Always, OnFailure, Never. Default to Always. More info: https://kubernetes.io/docs/concepts/workloads/pods/pod-lifecycle/#restart-policy  # noqa: E501

        :param restart_policy: The restart_policy of this V1beta1ExplainerSpec.  # noqa: E501
        :type: str
        """

        self._restart_policy = restart_policy

    @property
    def runtime_class_name(self):
        """Gets the runtime_class_name of this V1beta1ExplainerSpec.  # noqa: E501

        RuntimeClassName refers to a RuntimeClass object in the node.k8s.io group, which should be used to run this pod.  If no RuntimeClass resource matches the named class, the pod will not be run. If unset or empty, the \"legacy\" RuntimeClass will be used, which is an implicit class with an empty definition that uses the default runtime handler. More info: https://git.k8s.io/enhancements/keps/sig-node/585-runtime-class This is a beta feature as of Kubernetes v1.14.  # noqa: E501

        :return: The runtime_class_name of this V1beta1ExplainerSpec.  # noqa: E501
        :rtype: str
        """
        return self._runtime_class_name

    @runtime_class_name.setter
    def runtime_class_name(self, runtime_class_name):
        """Sets the runtime_class_name of this V1beta1ExplainerSpec.

        RuntimeClassName refers to a RuntimeClass object in the node.k8s.io group, which should be used to run this pod.  If no RuntimeClass resource matches the named class, the pod will not be run. If unset or empty, the \"legacy\" RuntimeClass will be used, which is an implicit class with an empty definition that uses the default runtime handler. More info: https://git.k8s.io/enhancements/keps/sig-node/585-runtime-class This is a beta feature as of Kubernetes v1.14.  # noqa: E501

        :param runtime_class_name: The runtime_class_name of this V1beta1ExplainerSpec.  # noqa: E501
        :type: str
        """

        self._runtime_class_name = runtime_class_name

    @property
    def scale_metric(self):
        """Gets the scale_metric of this V1beta1ExplainerSpec.  # noqa: E501

        ScaleMetric defines the scaling metric type watched by autoscaler possible values are concurrency, rps, cpu, memory. concurrency, rps are supported via Knative Pod Autoscaler(https://knative.dev/docs/serving/autoscaling/autoscaling-metrics).  # noqa: E501

        :return: The scale_metric of this V1beta1ExplainerSpec.  # noqa: E501
        :rtype: str
        """
        return self._scale_metric

    @scale_metric.setter
    def scale_metric(self, scale_metric):
        """Sets the scale_metric of this V1beta1ExplainerSpec.

        ScaleMetric defines the scaling metric type watched by autoscaler possible values are concurrency, rps, cpu, memory. concurrency, rps are supported via Knative Pod Autoscaler(https://knative.dev/docs/serving/autoscaling/autoscaling-metrics).  # noqa: E501

        :param scale_metric: The scale_metric of this V1beta1ExplainerSpec.  # noqa: E501
        :type: str
        """

        self._scale_metric = scale_metric

    @property
    def scale_target(self):
        """Gets the scale_target of this V1beta1ExplainerSpec.  # noqa: E501

        ScaleTarget specifies the integer target value of the metric type the Autoscaler watches for. concurrency and rps targets are supported by Knative Pod Autoscaler (https://knative.dev/docs/serving/autoscaling/autoscaling-targets/).  # noqa: E501

        :return: The scale_target of this V1beta1ExplainerSpec.  # noqa: E501
        :rtype: int
        """
        return self._scale_target

    @scale_target.setter
    def scale_target(self, scale_target):
        """Sets the scale_target of this V1beta1ExplainerSpec.

        ScaleTarget specifies the integer target value of the metric type the Autoscaler watches for. concurrency and rps targets are supported by Knative Pod Autoscaler (https://knative.dev/docs/serving/autoscaling/autoscaling-targets/).  # noqa: E501

        :param scale_target: The scale_target of this V1beta1ExplainerSpec.  # noqa: E501
        :type: int
        """

        self._scale_target = scale_target

    @property
    def scheduler_name(self):
        """Gets the scheduler_name of this V1beta1ExplainerSpec.  # noqa: E501

        If specified, the pod will be dispatched by specified scheduler. If not specified, the pod will be dispatched by default scheduler.  # noqa: E501

        :return: The scheduler_name of this V1beta1ExplainerSpec.  # noqa: E501
        :rtype: str
        """
        return self._scheduler_name

    @scheduler_name.setter
    def scheduler_name(self, scheduler_name):
        """Sets the scheduler_name of this V1beta1ExplainerSpec.

        If specified, the pod will be dispatched by specified scheduler. If not specified, the pod will be dispatched by default scheduler.  # noqa: E501

        :param scheduler_name: The scheduler_name of this V1beta1ExplainerSpec.  # noqa: E501
        :type: str
        """

        self._scheduler_name = scheduler_name

    @property
    def scheduling_gates(self):
        """Gets the scheduling_gates of this V1beta1ExplainerSpec.  # noqa: E501

        SchedulingGates is an opaque list of values that if specified will block scheduling the pod. If schedulingGates is not empty, the pod will stay in the SchedulingGated state and the scheduler will not attempt to schedule the pod.  SchedulingGates can only be set at pod creation time, and be removed only afterwards.  This is a beta feature enabled by the PodSchedulingReadiness feature gate.  # noqa: E501

        :return: The scheduling_gates of this V1beta1ExplainerSpec.  # noqa: E501
        :rtype: list[V1PodSchedulingGate]
        """
        return self._scheduling_gates

    @scheduling_gates.setter
    def scheduling_gates(self, scheduling_gates):
        """Sets the scheduling_gates of this V1beta1ExplainerSpec.

        SchedulingGates is an opaque list of values that if specified will block scheduling the pod. If schedulingGates is not empty, the pod will stay in the SchedulingGated state and the scheduler will not attempt to schedule the pod.  SchedulingGates can only be set at pod creation time, and be removed only afterwards.  This is a beta feature enabled by the PodSchedulingReadiness feature gate.  # noqa: E501

        :param scheduling_gates: The scheduling_gates of this V1beta1ExplainerSpec.  # noqa: E501
        :type: list[V1PodSchedulingGate]
        """

        self._scheduling_gates = scheduling_gates

    @property
    def security_context(self):
        """Gets the security_context of this V1beta1ExplainerSpec.  # noqa: E501


        :return: The security_context of this V1beta1ExplainerSpec.  # noqa: E501
        :rtype: V1PodSecurityContext
        """
        return self._security_context

    @security_context.setter
    def security_context(self, security_context):
        """Sets the security_context of this V1beta1ExplainerSpec.


        :param security_context: The security_context of this V1beta1ExplainerSpec.  # noqa: E501
        :type: V1PodSecurityContext
        """

        self._security_context = security_context

    @property
    def service_account(self):
        """Gets the service_account of this V1beta1ExplainerSpec.  # noqa: E501

        DeprecatedServiceAccount is a depreciated alias for ServiceAccountName. Deprecated: Use serviceAccountName instead.  # noqa: E501

        :return: The service_account of this V1beta1ExplainerSpec.  # noqa: E501
        :rtype: str
        """
        return self._service_account

    @service_account.setter
    def service_account(self, service_account):
        """Sets the service_account of this V1beta1ExplainerSpec.

        DeprecatedServiceAccount is a depreciated alias for ServiceAccountName. Deprecated: Use serviceAccountName instead.  # noqa: E501

        :param service_account: The service_account of this V1beta1ExplainerSpec.  # noqa: E501
        :type: str
        """

        self._service_account = service_account

    @property
    def service_account_name(self):
        """Gets the service_account_name of this V1beta1ExplainerSpec.  # noqa: E501

        ServiceAccountName is the name of the ServiceAccount to use to run this pod. More info: https://kubernetes.io/docs/tasks/configure-pod-container/configure-service-account/  # noqa: E501

        :return: The service_account_name of this V1beta1ExplainerSpec.  # noqa: E501
        :rtype: str
        """
        return self._service_account_name

    @service_account_name.setter
    def service_account_name(self, service_account_name):
        """Sets the service_account_name of this V1beta1ExplainerSpec.

        ServiceAccountName is the name of the ServiceAccount to use to run this pod. More info: https://kubernetes.io/docs/tasks/configure-pod-container/configure-service-account/  # noqa: E501

        :param service_account_name: The service_account_name of this V1beta1ExplainerSpec.  # noqa: E501
        :type: str
        """

        self._service_account_name = service_account_name

    @property
    def set_hostname_as_fqdn(self):
        """Gets the set_hostname_as_fqdn of this V1beta1ExplainerSpec.  # noqa: E501

        If true the pod's hostname will be configured as the pod's FQDN, rather than the leaf name (the default). In Linux containers, this means setting the FQDN in the hostname field of the kernel (the nodename field of struct utsname). In Windows containers, this means setting the registry value of hostname for the registry key HKEY_LOCAL_MACHINE\\\\SYSTEM\\\\CurrentControlSet\\\\Services\\\\Tcpip\\\\Parameters to FQDN. If a pod does not have FQDN, this has no effect. Default to false.  # noqa: E501

        :return: The set_hostname_as_fqdn of this V1beta1ExplainerSpec.  # noqa: E501
        :rtype: bool
        """
        return self._set_hostname_as_fqdn

    @set_hostname_as_fqdn.setter
    def set_hostname_as_fqdn(self, set_hostname_as_fqdn):
        """Sets the set_hostname_as_fqdn of this V1beta1ExplainerSpec.

        If true the pod's hostname will be configured as the pod's FQDN, rather than the leaf name (the default). In Linux containers, this means setting the FQDN in the hostname field of the kernel (the nodename field of struct utsname). In Windows containers, this means setting the registry value of hostname for the registry key HKEY_LOCAL_MACHINE\\\\SYSTEM\\\\CurrentControlSet\\\\Services\\\\Tcpip\\\\Parameters to FQDN. If a pod does not have FQDN, this has no effect. Default to false.  # noqa: E501

        :param set_hostname_as_fqdn: The set_hostname_as_fqdn of this V1beta1ExplainerSpec.  # noqa: E501
        :type: bool
        """

        self._set_hostname_as_fqdn = set_hostname_as_fqdn

    @property
    def share_process_namespace(self):
        """Gets the share_process_namespace of this V1beta1ExplainerSpec.  # noqa: E501

        Share a single process namespace between all of the containers in a pod. When this is set containers will be able to view and signal processes from other containers in the same pod, and the first process in each container will not be assigned PID 1. HostPID and ShareProcessNamespace cannot both be set. Optional: Default to false.  # noqa: E501

        :return: The share_process_namespace of this V1beta1ExplainerSpec.  # noqa: E501
        :rtype: bool
        """
        return self._share_process_namespace

    @share_process_namespace.setter
    def share_process_namespace(self, share_process_namespace):
        """Sets the share_process_namespace of this V1beta1ExplainerSpec.

        Share a single process namespace between all of the containers in a pod. When this is set containers will be able to view and signal processes from other containers in the same pod, and the first process in each container will not be assigned PID 1. HostPID and ShareProcessNamespace cannot both be set. Optional: Default to false.  # noqa: E501

        :param share_process_namespace: The share_process_namespace of this V1beta1ExplainerSpec.  # noqa: E501
        :type: bool
        """

        self._share_process_namespace = share_process_namespace

    @property
    def subdomain(self):
        """Gets the subdomain of this V1beta1ExplainerSpec.  # noqa: E501

        If specified, the fully qualified Pod hostname will be \"<hostname>.<subdomain>.<pod namespace>.svc.<cluster domain>\". If not specified, the pod will not have a domainname at all.  # noqa: E501

        :return: The subdomain of this V1beta1ExplainerSpec.  # noqa: E501
        :rtype: str
        """
        return self._subdomain

    @subdomain.setter
    def subdomain(self, subdomain):
        """Sets the subdomain of this V1beta1ExplainerSpec.

        If specified, the fully qualified Pod hostname will be \"<hostname>.<subdomain>.<pod namespace>.svc.<cluster domain>\". If not specified, the pod will not have a domainname at all.  # noqa: E501

        :param subdomain: The subdomain of this V1beta1ExplainerSpec.  # noqa: E501
        :type: str
        """

        self._subdomain = subdomain

    @property
    def termination_grace_period_seconds(self):
        """Gets the termination_grace_period_seconds of this V1beta1ExplainerSpec.  # noqa: E501

        Optional duration in seconds the pod needs to terminate gracefully. May be decreased in delete request. Value must be non-negative integer. The value zero indicates stop immediately via the kill signal (no opportunity to shut down). If this value is nil, the default grace period will be used instead. The grace period is the duration in seconds after the processes running in the pod are sent a termination signal and the time when the processes are forcibly halted with a kill signal. Set this value longer than the expected cleanup time for your process. Defaults to 30 seconds.  # noqa: E501

        :return: The termination_grace_period_seconds of this V1beta1ExplainerSpec.  # noqa: E501
        :rtype: int
        """
        return self._termination_grace_period_seconds

    @termination_grace_period_seconds.setter
    def termination_grace_period_seconds(self, termination_grace_period_seconds):
        """Sets the termination_grace_period_seconds of this V1beta1ExplainerSpec.

        Optional duration in seconds the pod needs to terminate gracefully. May be decreased in delete request. Value must be non-negative integer. The value zero indicates stop immediately via the kill signal (no opportunity to shut down). If this value is nil, the default grace period will be used instead. The grace period is the duration in seconds after the processes running in the pod are sent a termination signal and the time when the processes are forcibly halted with a kill signal. Set this value longer than the expected cleanup time for your process. Defaults to 30 seconds.  # noqa: E501

        :param termination_grace_period_seconds: The termination_grace_period_seconds of this V1beta1ExplainerSpec.  # noqa: E501
        :type: int
        """

        self._termination_grace_period_seconds = termination_grace_period_seconds

    @property
    def timeout(self):
        """Gets the timeout of this V1beta1ExplainerSpec.  # noqa: E501

        TimeoutSeconds specifies the number of seconds to wait before timing out a request to the component.  # noqa: E501

        :return: The timeout of this V1beta1ExplainerSpec.  # noqa: E501
        :rtype: int
        """
        return self._timeout

    @timeout.setter
    def timeout(self, timeout):
        """Sets the timeout of this V1beta1ExplainerSpec.

        TimeoutSeconds specifies the number of seconds to wait before timing out a request to the component.  # noqa: E501

        :param timeout: The timeout of this V1beta1ExplainerSpec.  # noqa: E501
        :type: int
        """

        self._timeout = timeout

    @property
    def tolerations(self):
        """Gets the tolerations of this V1beta1ExplainerSpec.  # noqa: E501

        If specified, the pod's tolerations.  # noqa: E501

        :return: The tolerations of this V1beta1ExplainerSpec.  # noqa: E501
        :rtype: list[V1Toleration]
        """
        return self._tolerations

    @tolerations.setter
    def tolerations(self, tolerations):
        """Sets the tolerations of this V1beta1ExplainerSpec.

        If specified, the pod's tolerations.  # noqa: E501

        :param tolerations: The tolerations of this V1beta1ExplainerSpec.  # noqa: E501
        :type: list[V1Toleration]
        """

        self._tolerations = tolerations

    @property
    def topology_spread_constraints(self):
        """Gets the topology_spread_constraints of this V1beta1ExplainerSpec.  # noqa: E501

        TopologySpreadConstraints describes how a group of pods ought to spread across topology domains. Scheduler will schedule pods in a way which abides by the constraints. All topologySpreadConstraints are ANDed.  # noqa: E501

        :return: The topology_spread_constraints of this V1beta1ExplainerSpec.  # noqa: E501
        :rtype: list[V1TopologySpreadConstraint]
        """
        return self._topology_spread_constraints

    @topology_spread_constraints.setter
    def topology_spread_constraints(self, topology_spread_constraints):
        """Sets the topology_spread_constraints of this V1beta1ExplainerSpec.

        TopologySpreadConstraints describes how a group of pods ought to spread across topology domains. Scheduler will schedule pods in a way which abides by the constraints. All topologySpreadConstraints are ANDed.  # noqa: E501

        :param topology_spread_constraints: The topology_spread_constraints of this V1beta1ExplainerSpec.  # noqa: E501
        :type: list[V1TopologySpreadConstraint]
        """

        self._topology_spread_constraints = topology_spread_constraints

    @property
    def volumes(self):
        """Gets the volumes of this V1beta1ExplainerSpec.  # noqa: E501

        List of volumes that can be mounted by containers belonging to the pod. More info: https://kubernetes.io/docs/concepts/storage/volumes  # noqa: E501

        :return: The volumes of this V1beta1ExplainerSpec.  # noqa: E501
        :rtype: list[V1Volume]
        """
        return self._volumes

    @volumes.setter
    def volumes(self, volumes):
        """Sets the volumes of this V1beta1ExplainerSpec.

        List of volumes that can be mounted by containers belonging to the pod. More info: https://kubernetes.io/docs/concepts/storage/volumes  # noqa: E501

        :param volumes: The volumes of this V1beta1ExplainerSpec.  # noqa: E501
        :type: list[V1Volume]
        """

        self._volumes = volumes

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, V1beta1ExplainerSpec):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, V1beta1ExplainerSpec):
            return True

        return self.to_dict() != other.to_dict()



# Copyright 2024 The KServe Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from typing import Union
from http import HTTPStatus
from fastapi.responses import JSONResponse
from kserve.logging import logger
from .types.openapi import Error, ErrorResponse


class OpenAIError(Exception):
    """
    Exception class for generic OpenAI error.
    """

    def __init__(self, response: Union[str, ErrorResponse]):
        self.response = response

    def __str__(self):
        return (
            self.response.error.message
            if isinstance(self.response, ErrorResponse)
            else self.response
        )


async def openai_error_handler(_, exc: OpenAIError):
    logger.error("Exception:", exc_info=exc)

    response = (
        exc.response
        if isinstance(exc.response, ErrorResponse)
        else create_error_response(
            message=str(exc),
            err_type=type(exc).__name__,
            status_code=HTTPStatus.INTERNAL_SERVER_ERROR,
        )
    )

    return JSONResponse(
        status_code=int(response.error.code),
        content=response.model_dump(),
    )


def create_error_response(
    message: str,
    err_type: str = "BadRequestError",
    param: str = "",
    status_code: HTTPStatus = HTTPStatus.BAD_REQUEST,
) -> ErrorResponse:
    error = Error(
        message=message, type=err_type, param=param, code=str(status_code.value)
    )
    return ErrorResponse(error=error)



# Copyright 2023 The KServe Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# coding: utf-8

"""
    KServe

    Python SDK for KServe  # noqa: E501

    The version of the OpenAPI document: v0.1
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from kserve.configuration import Configuration


class V1alpha1ServingRuntimePodSpec(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'affinity': 'V1Affinity',
        'annotations': 'dict(str, str)',
        'containers': 'list[V1Container]',
        'host_ipc': 'bool',
        'image_pull_secrets': 'list[V1LocalObjectReference]',
        'labels': 'dict(str, str)',
        'node_selector': 'dict(str, str)',
        'tolerations': 'list[V1Toleration]',
        'volumes': 'list[V1Volume]'
    }

    attribute_map = {
        'affinity': 'affinity',
        'annotations': 'annotations',
        'containers': 'containers',
        'host_ipc': 'hostIPC',
        'image_pull_secrets': 'imagePullSecrets',
        'labels': 'labels',
        'node_selector': 'nodeSelector',
        'tolerations': 'tolerations',
        'volumes': 'volumes'
    }

    def __init__(self, affinity=None, annotations=None, containers=None, host_ipc=None, image_pull_secrets=None, labels=None, node_selector=None, tolerations=None, volumes=None, local_vars_configuration=None):  # noqa: E501
        """V1alpha1ServingRuntimePodSpec - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._affinity = None
        self._annotations = None
        self._containers = None
        self._host_ipc = None
        self._image_pull_secrets = None
        self._labels = None
        self._node_selector = None
        self._tolerations = None
        self._volumes = None
        self.discriminator = None

        if affinity is not None:
            self.affinity = affinity
        if annotations is not None:
            self.annotations = annotations
        self.containers = containers
        if host_ipc is not None:
            self.host_ipc = host_ipc
        if image_pull_secrets is not None:
            self.image_pull_secrets = image_pull_secrets
        if labels is not None:
            self.labels = labels
        if node_selector is not None:
            self.node_selector = node_selector
        if tolerations is not None:
            self.tolerations = tolerations
        if volumes is not None:
            self.volumes = volumes

    @property
    def affinity(self):
        """Gets the affinity of this V1alpha1ServingRuntimePodSpec.  # noqa: E501


        :return: The affinity of this V1alpha1ServingRuntimePodSpec.  # noqa: E501
        :rtype: V1Affinity
        """
        return self._affinity

    @affinity.setter
    def affinity(self, affinity):
        """Sets the affinity of this V1alpha1ServingRuntimePodSpec.


        :param affinity: The affinity of this V1alpha1ServingRuntimePodSpec.  # noqa: E501
        :type: V1Affinity
        """

        self._affinity = affinity

    @property
    def annotations(self):
        """Gets the annotations of this V1alpha1ServingRuntimePodSpec.  # noqa: E501

        Annotations that will be add to the pod. More info: http://kubernetes.io/docs/user-guide/annotations  # noqa: E501

        :return: The annotations of this V1alpha1ServingRuntimePodSpec.  # noqa: E501
        :rtype: dict(str, str)
        """
        return self._annotations

    @annotations.setter
    def annotations(self, annotations):
        """Sets the annotations of this V1alpha1ServingRuntimePodSpec.

        Annotations that will be add to the pod. More info: http://kubernetes.io/docs/user-guide/annotations  # noqa: E501

        :param annotations: The annotations of this V1alpha1ServingRuntimePodSpec.  # noqa: E501
        :type: dict(str, str)
        """

        self._annotations = annotations

    @property
    def containers(self):
        """Gets the containers of this V1alpha1ServingRuntimePodSpec.  # noqa: E501

        List of containers belonging to the pod. Containers cannot currently be added or removed. There must be at least one container in a Pod. Cannot be updated.  # noqa: E501

        :return: The containers of this V1alpha1ServingRuntimePodSpec.  # noqa: E501
        :rtype: list[V1Container]
        """
        return self._containers

    @containers.setter
    def containers(self, containers):
        """Sets the containers of this V1alpha1ServingRuntimePodSpec.

        List of containers belonging to the pod. Containers cannot currently be added or removed. There must be at least one container in a Pod. Cannot be updated.  # noqa: E501

        :param containers: The containers of this V1alpha1ServingRuntimePodSpec.  # noqa: E501
        :type: list[V1Container]
        """
        if self.local_vars_configuration.client_side_validation and containers is None:  # noqa: E501
            raise ValueError("Invalid value for `containers`, must not be `None`")  # noqa: E501

        self._containers = containers

    @property
    def host_ipc(self):
        """Gets the host_ipc of this V1alpha1ServingRuntimePodSpec.  # noqa: E501

        Use the host's ipc namespace. Optional: Default to false.  # noqa: E501

        :return: The host_ipc of this V1alpha1ServingRuntimePodSpec.  # noqa: E501
        :rtype: bool
        """
        return self._host_ipc

    @host_ipc.setter
    def host_ipc(self, host_ipc):
        """Sets the host_ipc of this V1alpha1ServingRuntimePodSpec.

        Use the host's ipc namespace. Optional: Default to false.  # noqa: E501

        :param host_ipc: The host_ipc of this V1alpha1ServingRuntimePodSpec.  # noqa: E501
        :type: bool
        """

        self._host_ipc = host_ipc

    @property
    def image_pull_secrets(self):
        """Gets the image_pull_secrets of this V1alpha1ServingRuntimePodSpec.  # noqa: E501

        ImagePullSecrets is an optional list of references to secrets in the same namespace to use for pulling any of the images used by this PodSpec. If specified, these secrets will be passed to individual puller implementations for them to use. For example, in the case of docker, only DockerConfig type secrets are honored. More info: https://kubernetes.io/docs/concepts/containers/images#specifying-imagepullsecrets-on-a-pod  # noqa: E501

        :return: The image_pull_secrets of this V1alpha1ServingRuntimePodSpec.  # noqa: E501
        :rtype: list[V1LocalObjectReference]
        """
        return self._image_pull_secrets

    @image_pull_secrets.setter
    def image_pull_secrets(self, image_pull_secrets):
        """Sets the image_pull_secrets of this V1alpha1ServingRuntimePodSpec.

        ImagePullSecrets is an optional list of references to secrets in the same namespace to use for pulling any of the images used by this PodSpec. If specified, these secrets will be passed to individual puller implementations for them to use. For example, in the case of docker, only DockerConfig type secrets are honored. More info: https://kubernetes.io/docs/concepts/containers/images#specifying-imagepullsecrets-on-a-pod  # noqa: E501

        :param image_pull_secrets: The image_pull_secrets of this V1alpha1ServingRuntimePodSpec.  # noqa: E501
        :type: list[V1LocalObjectReference]
        """

        self._image_pull_secrets = image_pull_secrets

    @property
    def labels(self):
        """Gets the labels of this V1alpha1ServingRuntimePodSpec.  # noqa: E501

        Labels that will be add to the pod. More info: http://kubernetes.io/docs/user-guide/labels  # noqa: E501

        :return: The labels of this V1alpha1ServingRuntimePodSpec.  # noqa: E501
        :rtype: dict(str, str)
        """
        return self._labels

    @labels.setter
    def labels(self, labels):
        """Sets the labels of this V1alpha1ServingRuntimePodSpec.

        Labels that will be add to the pod. More info: http://kubernetes.io/docs/user-guide/labels  # noqa: E501

        :param labels: The labels of this V1alpha1ServingRuntimePodSpec.  # noqa: E501
        :type: dict(str, str)
        """

        self._labels = labels

    @property
    def node_selector(self):
        """Gets the node_selector of this V1alpha1ServingRuntimePodSpec.  # noqa: E501

        NodeSelector is a selector which must be true for the pod to fit on a node. Selector which must match a node's labels for the pod to be scheduled on that node. More info: https://kubernetes.io/docs/concepts/configuration/assign-pod-node/  # noqa: E501

        :return: The node_selector of this V1alpha1ServingRuntimePodSpec.  # noqa: E501
        :rtype: dict(str, str)
        """
        return self._node_selector

    @node_selector.setter
    def node_selector(self, node_selector):
        """Sets the node_selector of this V1alpha1ServingRuntimePodSpec.

        NodeSelector is a selector which must be true for the pod to fit on a node. Selector which must match a node's labels for the pod to be scheduled on that node. More info: https://kubernetes.io/docs/concepts/configuration/assign-pod-node/  # noqa: E501

        :param node_selector: The node_selector of this V1alpha1ServingRuntimePodSpec.  # noqa: E501
        :type: dict(str, str)
        """

        self._node_selector = node_selector

    @property
    def tolerations(self):
        """Gets the tolerations of this V1alpha1ServingRuntimePodSpec.  # noqa: E501

        If specified, the pod's tolerations.  # noqa: E501

        :return: The tolerations of this V1alpha1ServingRuntimePodSpec.  # noqa: E501
        :rtype: list[V1Toleration]
        """
        return self._tolerations

    @tolerations.setter
    def tolerations(self, tolerations):
        """Sets the tolerations of this V1alpha1ServingRuntimePodSpec.

        If specified, the pod's tolerations.  # noqa: E501

        :param tolerations: The tolerations of this V1alpha1ServingRuntimePodSpec.  # noqa: E501
        :type: list[V1Toleration]
        """

        self._tolerations = tolerations

    @property
    def volumes(self):
        """Gets the volumes of this V1alpha1ServingRuntimePodSpec.  # noqa: E501

        List of volumes that can be mounted by containers belonging to the pod. More info: https://kubernetes.io/docs/concepts/storage/volumes  # noqa: E501

        :return: The volumes of this V1alpha1ServingRuntimePodSpec.  # noqa: E501
        :rtype: list[V1Volume]
        """
        return self._volumes

    @volumes.setter
    def volumes(self, volumes):
        """Sets the volumes of this V1alpha1ServingRuntimePodSpec.

        List of volumes that can be mounted by containers belonging to the pod. More info: https://kubernetes.io/docs/concepts/storage/volumes  # noqa: E501

        :param volumes: The volumes of this V1alpha1ServingRuntimePodSpec.  # noqa: E501
        :type: list[V1Volume]
        """

        self._volumes = volumes

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, V1alpha1ServingRuntimePodSpec):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, V1alpha1ServingRuntimePodSpec):
            return True

        return self.to_dict() != other.to_dict()



# Copyright 2023 The KServe Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# coding: utf-8

"""
    KServe

    Python SDK for KServe  # noqa: E501

    The version of the OpenAPI document: v0.1
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from kserve.configuration import Configuration


class V1beta1LocalModelConfig(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'default_job_image': 'str',
        'disable_volume_management': 'bool',
        'enabled': 'bool',
        'fs_group': 'int',
        'job_namespace': 'str',
        'job_ttl_seconds_after_finished': 'int',
        'reconcilation_frequency_in_secs': 'int'
    }

    attribute_map = {
        'default_job_image': 'defaultJobImage',
        'disable_volume_management': 'disableVolumeManagement',
        'enabled': 'enabled',
        'fs_group': 'fsGroup',
        'job_namespace': 'jobNamespace',
        'job_ttl_seconds_after_finished': 'jobTTLSecondsAfterFinished',
        'reconcilation_frequency_in_secs': 'reconcilationFrequencyInSecs'
    }

    def __init__(self, default_job_image=None, disable_volume_management=None, enabled=False, fs_group=None, job_namespace='', job_ttl_seconds_after_finished=None, reconcilation_frequency_in_secs=None, local_vars_configuration=None):  # noqa: E501
        """V1beta1LocalModelConfig - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._default_job_image = None
        self._disable_volume_management = None
        self._enabled = None
        self._fs_group = None
        self._job_namespace = None
        self._job_ttl_seconds_after_finished = None
        self._reconcilation_frequency_in_secs = None
        self.discriminator = None

        if default_job_image is not None:
            self.default_job_image = default_job_image
        if disable_volume_management is not None:
            self.disable_volume_management = disable_volume_management
        self.enabled = enabled
        if fs_group is not None:
            self.fs_group = fs_group
        self.job_namespace = job_namespace
        if job_ttl_seconds_after_finished is not None:
            self.job_ttl_seconds_after_finished = job_ttl_seconds_after_finished
        if reconcilation_frequency_in_secs is not None:
            self.reconcilation_frequency_in_secs = reconcilation_frequency_in_secs

    @property
    def default_job_image(self):
        """Gets the default_job_image of this V1beta1LocalModelConfig.  # noqa: E501


        :return: The default_job_image of this V1beta1LocalModelConfig.  # noqa: E501
        :rtype: str
        """
        return self._default_job_image

    @default_job_image.setter
    def default_job_image(self, default_job_image):
        """Sets the default_job_image of this V1beta1LocalModelConfig.


        :param default_job_image: The default_job_image of this V1beta1LocalModelConfig.  # noqa: E501
        :type: str
        """

        self._default_job_image = default_job_image

    @property
    def disable_volume_management(self):
        """Gets the disable_volume_management of this V1beta1LocalModelConfig.  # noqa: E501


        :return: The disable_volume_management of this V1beta1LocalModelConfig.  # noqa: E501
        :rtype: bool
        """
        return self._disable_volume_management

    @disable_volume_management.setter
    def disable_volume_management(self, disable_volume_management):
        """Sets the disable_volume_management of this V1beta1LocalModelConfig.


        :param disable_volume_management: The disable_volume_management of this V1beta1LocalModelConfig.  # noqa: E501
        :type: bool
        """

        self._disable_volume_management = disable_volume_management

    @property
    def enabled(self):
        """Gets the enabled of this V1beta1LocalModelConfig.  # noqa: E501


        :return: The enabled of this V1beta1LocalModelConfig.  # noqa: E501
        :rtype: bool
        """
        return self._enabled

    @enabled.setter
    def enabled(self, enabled):
        """Sets the enabled of this V1beta1LocalModelConfig.


        :param enabled: The enabled of this V1beta1LocalModelConfig.  # noqa: E501
        :type: bool
        """
        if self.local_vars_configuration.client_side_validation and enabled is None:  # noqa: E501
            raise ValueError("Invalid value for `enabled`, must not be `None`")  # noqa: E501

        self._enabled = enabled

    @property
    def fs_group(self):
        """Gets the fs_group of this V1beta1LocalModelConfig.  # noqa: E501


        :return: The fs_group of this V1beta1LocalModelConfig.  # noqa: E501
        :rtype: int
        """
        return self._fs_group

    @fs_group.setter
    def fs_group(self, fs_group):
        """Sets the fs_group of this V1beta1LocalModelConfig.


        :param fs_group: The fs_group of this V1beta1LocalModelConfig.  # noqa: E501
        :type: int
        """

        self._fs_group = fs_group

    @property
    def job_namespace(self):
        """Gets the job_namespace of this V1beta1LocalModelConfig.  # noqa: E501


        :return: The job_namespace of this V1beta1LocalModelConfig.  # noqa: E501
        :rtype: str
        """
        return self._job_namespace

    @job_namespace.setter
    def job_namespace(self, job_namespace):
        """Sets the job_namespace of this V1beta1LocalModelConfig.


        :param job_namespace: The job_namespace of this V1beta1LocalModelConfig.  # noqa: E501
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and job_namespace is None:  # noqa: E501
            raise ValueError("Invalid value for `job_namespace`, must not be `None`")  # noqa: E501

        self._job_namespace = job_namespace

    @property
    def job_ttl_seconds_after_finished(self):
        """Gets the job_ttl_seconds_after_finished of this V1beta1LocalModelConfig.  # noqa: E501


        :return: The job_ttl_seconds_after_finished of this V1beta1LocalModelConfig.  # noqa: E501
        :rtype: int
        """
        return self._job_ttl_seconds_after_finished

    @job_ttl_seconds_after_finished.setter
    def job_ttl_seconds_after_finished(self, job_ttl_seconds_after_finished):
        """Sets the job_ttl_seconds_after_finished of this V1beta1LocalModelConfig.


        :param job_ttl_seconds_after_finished: The job_ttl_seconds_after_finished of this V1beta1LocalModelConfig.  # noqa: E501
        :type: int
        """

        self._job_ttl_seconds_after_finished = job_ttl_seconds_after_finished

    @property
    def reconcilation_frequency_in_secs(self):
        """Gets the reconcilation_frequency_in_secs of this V1beta1LocalModelConfig.  # noqa: E501


        :return: The reconcilation_frequency_in_secs of this V1beta1LocalModelConfig.  # noqa: E501
        :rtype: int
        """
        return self._reconcilation_frequency_in_secs

    @reconcilation_frequency_in_secs.setter
    def reconcilation_frequency_in_secs(self, reconcilation_frequency_in_secs):
        """Sets the reconcilation_frequency_in_secs of this V1beta1LocalModelConfig.


        :param reconcilation_frequency_in_secs: The reconcilation_frequency_in_secs of this V1beta1LocalModelConfig.  # noqa: E501
        :type: int
        """

        self._reconcilation_frequency_in_secs = reconcilation_frequency_in_secs

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, V1beta1LocalModelConfig):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, V1beta1LocalModelConfig):
            return True

        return self.to_dict() != other.to_dict()



# Copyright 2023 The KServe Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# coding: utf-8

"""
    KServe

    Python SDK for KServe  # noqa: E501

    The version of the OpenAPI document: v0.1
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from kserve.configuration import Configuration


class V1beta1FailureInfo(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'exit_code': 'int',
        'location': 'str',
        'message': 'str',
        'model_revision_name': 'str',
        'reason': 'str',
        'time': 'V1Time'
    }

    attribute_map = {
        'exit_code': 'exitCode',
        'location': 'location',
        'message': 'message',
        'model_revision_name': 'modelRevisionName',
        'reason': 'reason',
        'time': 'time'
    }

    def __init__(self, exit_code=None, location=None, message=None, model_revision_name=None, reason=None, time=None, local_vars_configuration=None):  # noqa: E501
        """V1beta1FailureInfo - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._exit_code = None
        self._location = None
        self._message = None
        self._model_revision_name = None
        self._reason = None
        self._time = None
        self.discriminator = None

        if exit_code is not None:
            self.exit_code = exit_code
        if location is not None:
            self.location = location
        if message is not None:
            self.message = message
        if model_revision_name is not None:
            self.model_revision_name = model_revision_name
        if reason is not None:
            self.reason = reason
        if time is not None:
            self.time = time

    @property
    def exit_code(self):
        """Gets the exit_code of this V1beta1FailureInfo.  # noqa: E501

        Exit status from the last termination of the container  # noqa: E501

        :return: The exit_code of this V1beta1FailureInfo.  # noqa: E501
        :rtype: int
        """
        return self._exit_code

    @exit_code.setter
    def exit_code(self, exit_code):
        """Sets the exit_code of this V1beta1FailureInfo.

        Exit status from the last termination of the container  # noqa: E501

        :param exit_code: The exit_code of this V1beta1FailureInfo.  # noqa: E501
        :type: int
        """

        self._exit_code = exit_code

    @property
    def location(self):
        """Gets the location of this V1beta1FailureInfo.  # noqa: E501

        Name of component to which the failure relates (usually Pod name)  # noqa: E501

        :return: The location of this V1beta1FailureInfo.  # noqa: E501
        :rtype: str
        """
        return self._location

    @location.setter
    def location(self, location):
        """Sets the location of this V1beta1FailureInfo.

        Name of component to which the failure relates (usually Pod name)  # noqa: E501

        :param location: The location of this V1beta1FailureInfo.  # noqa: E501
        :type: str
        """

        self._location = location

    @property
    def message(self):
        """Gets the message of this V1beta1FailureInfo.  # noqa: E501

        Detailed error message  # noqa: E501

        :return: The message of this V1beta1FailureInfo.  # noqa: E501
        :rtype: str
        """
        return self._message

    @message.setter
    def message(self, message):
        """Sets the message of this V1beta1FailureInfo.

        Detailed error message  # noqa: E501

        :param message: The message of this V1beta1FailureInfo.  # noqa: E501
        :type: str
        """

        self._message = message

    @property
    def model_revision_name(self):
        """Gets the model_revision_name of this V1beta1FailureInfo.  # noqa: E501

        Internal Revision/ID of model, tied to specific Spec contents  # noqa: E501

        :return: The model_revision_name of this V1beta1FailureInfo.  # noqa: E501
        :rtype: str
        """
        return self._model_revision_name

    @model_revision_name.setter
    def model_revision_name(self, model_revision_name):
        """Sets the model_revision_name of this V1beta1FailureInfo.

        Internal Revision/ID of model, tied to specific Spec contents  # noqa: E501

        :param model_revision_name: The model_revision_name of this V1beta1FailureInfo.  # noqa: E501
        :type: str
        """

        self._model_revision_name = model_revision_name

    @property
    def reason(self):
        """Gets the reason of this V1beta1FailureInfo.  # noqa: E501

        High level class of failure  # noqa: E501

        :return: The reason of this V1beta1FailureInfo.  # noqa: E501
        :rtype: str
        """
        return self._reason

    @reason.setter
    def reason(self, reason):
        """Sets the reason of this V1beta1FailureInfo.

        High level class of failure  # noqa: E501

        :param reason: The reason of this V1beta1FailureInfo.  # noqa: E501
        :type: str
        """

        self._reason = reason

    @property
    def time(self):
        """Gets the time of this V1beta1FailureInfo.  # noqa: E501


        :return: The time of this V1beta1FailureInfo.  # noqa: E501
        :rtype: V1Time
        """
        return self._time

    @time.setter
    def time(self, time):
        """Sets the time of this V1beta1FailureInfo.


        :param time: The time of this V1beta1FailureInfo.  # noqa: E501
        :type: V1Time
        """

        self._time = time

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, V1beta1FailureInfo):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, V1beta1FailureInfo):
            return True

        return self.to_dict() != other.to_dict()



# Copyright 2022 The KServe Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from http import HTTPStatus
from .logging import logger
from fastapi.responses import JSONResponse


class ModelMissingError(Exception):
    def __init__(self, path):
        self.path = path

    def __str__(self):
        return self.path


class InferenceError(RuntimeError):
    def __init__(self, reason, status=None, debug_details=None):
        self.reason = reason
        self.status = status
        self.debug_details = debug_details

    def __str__(self):
        msg = super().__str__() if self.reason is None else self.reason
        if self.status is not None:
            msg = "[" + self.status + "] " + msg
        return msg


class InvalidInput(ValueError):
    """
    Exception class indicating invalid input arguments.
    HTTP Servers should return HTTP_400 (Bad Request).
    """

    def __init__(self, reason):
        self.reason = reason

    def __str__(self):
        return self.reason


class ModelNotFound(Exception):
    """
    Exception class indicating requested model does not exist.
    HTTP Servers should return HTTP_404 (Not Found).
    """

    def __init__(self, model_name=None):
        self.reason = f"Model with name {model_name} does not exist."

    def __str__(self):
        return self.reason


class WorkersShouldBeLessThanMaxWorkersError(Exception):
    """
    Exception class indicating provided workers greater than the maximum workers allowed.
    HTTP Servers should return HTTP_404 (Not Found).
    """

    def __init__(self, max_workers=None):
        self.reason = f"Workers cannot be greater than {max_workers}"

    def __str__(self):
        return self.reason


class ModelNotReady(RuntimeError):
    def __init__(self, model_name: str, detail: str = None):
        self.model_name = model_name
        self.error_msg = f"Model with name {self.model_name} is not ready."
        if detail:
            self.error_msg = self.error_msg + " " + detail

    def __str__(self):
        return self.error_msg


class UnsupportedProtocol(Exception):
    """
    Exception class indicating requested protocol is not supported.
    """

    def __init__(self, protocol_version=None):
        self.reason = f"Unsupported protocol {protocol_version}."

    def __str__(self):
        return self.reason


class ServerNotReady(RuntimeError):
    def __init__(self, detail: str = None):
        self.error_msg = "Server is not ready."
        if detail:
            self.error_msg = self.error_msg + " " + detail

    def __str__(self):
        return self.error_msg


class ServerNotLive(RuntimeError):
    def __init__(self, detail: str = None):
        self.error_msg = "Server is not live."
        if detail:
            self.error_msg = self.error_msg + " " + detail

    def __str__(self):
        return self.error_msg


async def exception_handler(_, exc):
    logger.error("Exception:", exc_info=exc)
    return JSONResponse(
        status_code=HTTPStatus.INTERNAL_SERVER_ERROR, content={"error": str(exc)}
    )


async def invalid_input_handler(_, exc):
    logger.error("Exception:", exc_info=exc)
    return JSONResponse(status_code=HTTPStatus.BAD_REQUEST, content={"error": str(exc)})


async def inference_error_handler(_, exc):
    logger.error("Exception:", exc_info=exc)
    return JSONResponse(
        status_code=HTTPStatus.INTERNAL_SERVER_ERROR, content={"error": str(exc)}
    )


async def generic_exception_handler(_, exc):
    logger.error("Exception:", exc_info=exc)
    return JSONResponse(
        status_code=HTTPStatus.INTERNAL_SERVER_ERROR,
        content={"error": f"{type(exc).__name__} : {str(exc)}"},
    )


async def model_not_found_handler(_, exc):
    logger.error("Exception:", exc_info=exc)
    return JSONResponse(status_code=HTTPStatus.NOT_FOUND, content={"error": str(exc)})


async def model_not_ready_handler(_, exc):
    logger.error("Exception:", exc_info=exc)
    return JSONResponse(
        status_code=HTTPStatus.SERVICE_UNAVAILABLE, content={"error": str(exc)}
    )


async def not_implemented_error_handler(_, exc):
    logger.error("Exception:", exc_info=exc)
    return JSONResponse(
        status_code=HTTPStatus.NOT_IMPLEMENTED, content={"error": str(exc)}
    )


async def unsupported_protocol_error_handler(_, exc):
    logger.error("Exception:", exc_info=exc)
    return JSONResponse(
        status_code=HTTPStatus.NOT_IMPLEMENTED, content={"error": str(exc)}
    )


async def server_not_ready_handler(_, exc):
    logger.error("Exception:", exc_info=exc)
    return JSONResponse(
        status_code=HTTPStatus.SERVICE_UNAVAILABLE, content={"error": str(exc)}
    )


async def server_not_live_handler(_, exc):
    logger.error("Exception:", exc_info=exc)
    return JSONResponse(
        status_code=HTTPStatus.SERVICE_UNAVAILABLE, content={"error": str(exc)}
    )


class NoModelReady(RuntimeError):
    def __init__(self, models: [], detail: str = None):
        self.models = models
        self.detail = detail

    def __str__(self):
        model_name_list = [model.name for model in self.models]
        if len(model_name_list) == 1:
            self.error_msg = f"Model with name {model_name_list[0]} is not ready."
        else:
            self.error_msg = (
                f"Models with names {','.join(model_name_list)} are not ready."
            )
        if self.detail:
            self.error_msg = self.error_msg + " " + self.detail
        return self.error_msg



# Copyright 2023 The KServe Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# coding: utf-8

"""
    KServe

    Python SDK for KServe  # noqa: E501

    The version of the OpenAPI document: v0.1
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from kserve.configuration import Configuration


class V1beta1ComponentExtensionSpec(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'annotations': 'dict(str, str)',
        'batcher': 'V1beta1Batcher',
        'canary_traffic_percent': 'int',
        'container_concurrency': 'int',
        'deployment_strategy': 'K8sIoApiAppsV1DeploymentStrategy',
        'labels': 'dict(str, str)',
        'logger': 'V1beta1LoggerSpec',
        'max_replicas': 'int',
        'min_replicas': 'int',
        'scale_metric': 'str',
        'scale_target': 'int',
        'timeout': 'int'
    }

    attribute_map = {
        'annotations': 'annotations',
        'batcher': 'batcher',
        'canary_traffic_percent': 'canaryTrafficPercent',
        'container_concurrency': 'containerConcurrency',
        'deployment_strategy': 'deploymentStrategy',
        'labels': 'labels',
        'logger': 'logger',
        'max_replicas': 'maxReplicas',
        'min_replicas': 'minReplicas',
        'scale_metric': 'scaleMetric',
        'scale_target': 'scaleTarget',
        'timeout': 'timeout'
    }

    def __init__(self, annotations=None, batcher=None, canary_traffic_percent=None, container_concurrency=None, deployment_strategy=None, labels=None, logger=None, max_replicas=None, min_replicas=None, scale_metric=None, scale_target=None, timeout=None, local_vars_configuration=None):  # noqa: E501
        """V1beta1ComponentExtensionSpec - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._annotations = None
        self._batcher = None
        self._canary_traffic_percent = None
        self._container_concurrency = None
        self._deployment_strategy = None
        self._labels = None
        self._logger = None
        self._max_replicas = None
        self._min_replicas = None
        self._scale_metric = None
        self._scale_target = None
        self._timeout = None
        self.discriminator = None

        if annotations is not None:
            self.annotations = annotations
        if batcher is not None:
            self.batcher = batcher
        if canary_traffic_percent is not None:
            self.canary_traffic_percent = canary_traffic_percent
        if container_concurrency is not None:
            self.container_concurrency = container_concurrency
        if deployment_strategy is not None:
            self.deployment_strategy = deployment_strategy
        if labels is not None:
            self.labels = labels
        if logger is not None:
            self.logger = logger
        if max_replicas is not None:
            self.max_replicas = max_replicas
        if min_replicas is not None:
            self.min_replicas = min_replicas
        if scale_metric is not None:
            self.scale_metric = scale_metric
        if scale_target is not None:
            self.scale_target = scale_target
        if timeout is not None:
            self.timeout = timeout

    @property
    def annotations(self):
        """Gets the annotations of this V1beta1ComponentExtensionSpec.  # noqa: E501

        Annotations that will be added to the component pod. More info: https://kubernetes.io/docs/concepts/overview/working-with-objects/annotations/  # noqa: E501

        :return: The annotations of this V1beta1ComponentExtensionSpec.  # noqa: E501
        :rtype: dict(str, str)
        """
        return self._annotations

    @annotations.setter
    def annotations(self, annotations):
        """Sets the annotations of this V1beta1ComponentExtensionSpec.

        Annotations that will be added to the component pod. More info: https://kubernetes.io/docs/concepts/overview/working-with-objects/annotations/  # noqa: E501

        :param annotations: The annotations of this V1beta1ComponentExtensionSpec.  # noqa: E501
        :type: dict(str, str)
        """

        self._annotations = annotations

    @property
    def batcher(self):
        """Gets the batcher of this V1beta1ComponentExtensionSpec.  # noqa: E501


        :return: The batcher of this V1beta1ComponentExtensionSpec.  # noqa: E501
        :rtype: V1beta1Batcher
        """
        return self._batcher

    @batcher.setter
    def batcher(self, batcher):
        """Sets the batcher of this V1beta1ComponentExtensionSpec.


        :param batcher: The batcher of this V1beta1ComponentExtensionSpec.  # noqa: E501
        :type: V1beta1Batcher
        """

        self._batcher = batcher

    @property
    def canary_traffic_percent(self):
        """Gets the canary_traffic_percent of this V1beta1ComponentExtensionSpec.  # noqa: E501

        CanaryTrafficPercent defines the traffic split percentage between the candidate revision and the last ready revision  # noqa: E501

        :return: The canary_traffic_percent of this V1beta1ComponentExtensionSpec.  # noqa: E501
        :rtype: int
        """
        return self._canary_traffic_percent

    @canary_traffic_percent.setter
    def canary_traffic_percent(self, canary_traffic_percent):
        """Sets the canary_traffic_percent of this V1beta1ComponentExtensionSpec.

        CanaryTrafficPercent defines the traffic split percentage between the candidate revision and the last ready revision  # noqa: E501

        :param canary_traffic_percent: The canary_traffic_percent of this V1beta1ComponentExtensionSpec.  # noqa: E501
        :type: int
        """

        self._canary_traffic_percent = canary_traffic_percent

    @property
    def container_concurrency(self):
        """Gets the container_concurrency of this V1beta1ComponentExtensionSpec.  # noqa: E501

        ContainerConcurrency specifies how many requests can be processed concurrently, this sets the hard limit of the container concurrency(https://knative.dev/docs/serving/autoscaling/concurrency).  # noqa: E501

        :return: The container_concurrency of this V1beta1ComponentExtensionSpec.  # noqa: E501
        :rtype: int
        """
        return self._container_concurrency

    @container_concurrency.setter
    def container_concurrency(self, container_concurrency):
        """Sets the container_concurrency of this V1beta1ComponentExtensionSpec.

        ContainerConcurrency specifies how many requests can be processed concurrently, this sets the hard limit of the container concurrency(https://knative.dev/docs/serving/autoscaling/concurrency).  # noqa: E501

        :param container_concurrency: The container_concurrency of this V1beta1ComponentExtensionSpec.  # noqa: E501
        :type: int
        """

        self._container_concurrency = container_concurrency

    @property
    def deployment_strategy(self):
        """Gets the deployment_strategy of this V1beta1ComponentExtensionSpec.  # noqa: E501


        :return: The deployment_strategy of this V1beta1ComponentExtensionSpec.  # noqa: E501
        :rtype: K8sIoApiAppsV1DeploymentStrategy
        """
        return self._deployment_strategy

    @deployment_strategy.setter
    def deployment_strategy(self, deployment_strategy):
        """Sets the deployment_strategy of this V1beta1ComponentExtensionSpec.


        :param deployment_strategy: The deployment_strategy of this V1beta1ComponentExtensionSpec.  # noqa: E501
        :type: K8sIoApiAppsV1DeploymentStrategy
        """

        self._deployment_strategy = deployment_strategy

    @property
    def labels(self):
        """Gets the labels of this V1beta1ComponentExtensionSpec.  # noqa: E501

        Labels that will be added to the component pod. More info: https://kubernetes.io/docs/concepts/overview/working-with-objects/labels/  # noqa: E501

        :return: The labels of this V1beta1ComponentExtensionSpec.  # noqa: E501
        :rtype: dict(str, str)
        """
        return self._labels

    @labels.setter
    def labels(self, labels):
        """Sets the labels of this V1beta1ComponentExtensionSpec.

        Labels that will be added to the component pod. More info: https://kubernetes.io/docs/concepts/overview/working-with-objects/labels/  # noqa: E501

        :param labels: The labels of this V1beta1ComponentExtensionSpec.  # noqa: E501
        :type: dict(str, str)
        """

        self._labels = labels

    @property
    def logger(self):
        """Gets the logger of this V1beta1ComponentExtensionSpec.  # noqa: E501


        :return: The logger of this V1beta1ComponentExtensionSpec.  # noqa: E501
        :rtype: V1beta1LoggerSpec
        """
        return self._logger

    @logger.setter
    def logger(self, logger):
        """Sets the logger of this V1beta1ComponentExtensionSpec.


        :param logger: The logger of this V1beta1ComponentExtensionSpec.  # noqa: E501
        :type: V1beta1LoggerSpec
        """

        self._logger = logger

    @property
    def max_replicas(self):
        """Gets the max_replicas of this V1beta1ComponentExtensionSpec.  # noqa: E501

        Maximum number of replicas for autoscaling.  # noqa: E501

        :return: The max_replicas of this V1beta1ComponentExtensionSpec.  # noqa: E501
        :rtype: int
        """
        return self._max_replicas

    @max_replicas.setter
    def max_replicas(self, max_replicas):
        """Sets the max_replicas of this V1beta1ComponentExtensionSpec.

        Maximum number of replicas for autoscaling.  # noqa: E501

        :param max_replicas: The max_replicas of this V1beta1ComponentExtensionSpec.  # noqa: E501
        :type: int
        """

        self._max_replicas = max_replicas

    @property
    def min_replicas(self):
        """Gets the min_replicas of this V1beta1ComponentExtensionSpec.  # noqa: E501

        Minimum number of replicas, defaults to 1 but can be set to 0 to enable scale-to-zero.  # noqa: E501

        :return: The min_replicas of this V1beta1ComponentExtensionSpec.  # noqa: E501
        :rtype: int
        """
        return self._min_replicas

    @min_replicas.setter
    def min_replicas(self, min_replicas):
        """Sets the min_replicas of this V1beta1ComponentExtensionSpec.

        Minimum number of replicas, defaults to 1 but can be set to 0 to enable scale-to-zero.  # noqa: E501

        :param min_replicas: The min_replicas of this V1beta1ComponentExtensionSpec.  # noqa: E501
        :type: int
        """

        self._min_replicas = min_replicas

    @property
    def scale_metric(self):
        """Gets the scale_metric of this V1beta1ComponentExtensionSpec.  # noqa: E501

        ScaleMetric defines the scaling metric type watched by autoscaler possible values are concurrency, rps, cpu, memory. concurrency, rps are supported via Knative Pod Autoscaler(https://knative.dev/docs/serving/autoscaling/autoscaling-metrics).  # noqa: E501

        :return: The scale_metric of this V1beta1ComponentExtensionSpec.  # noqa: E501
        :rtype: str
        """
        return self._scale_metric

    @scale_metric.setter
    def scale_metric(self, scale_metric):
        """Sets the scale_metric of this V1beta1ComponentExtensionSpec.

        ScaleMetric defines the scaling metric type watched by autoscaler possible values are concurrency, rps, cpu, memory. concurrency, rps are supported via Knative Pod Autoscaler(https://knative.dev/docs/serving/autoscaling/autoscaling-metrics).  # noqa: E501

        :param scale_metric: The scale_metric of this V1beta1ComponentExtensionSpec.  # noqa: E501
        :type: str
        """

        self._scale_metric = scale_metric

    @property
    def scale_target(self):
        """Gets the scale_target of this V1beta1ComponentExtensionSpec.  # noqa: E501

        ScaleTarget specifies the integer target value of the metric type the Autoscaler watches for. concurrency and rps targets are supported by Knative Pod Autoscaler (https://knative.dev/docs/serving/autoscaling/autoscaling-targets/).  # noqa: E501

        :return: The scale_target of this V1beta1ComponentExtensionSpec.  # noqa: E501
        :rtype: int
        """
        return self._scale_target

    @scale_target.setter
    def scale_target(self, scale_target):
        """Sets the scale_target of this V1beta1ComponentExtensionSpec.

        ScaleTarget specifies the integer target value of the metric type the Autoscaler watches for. concurrency and rps targets are supported by Knative Pod Autoscaler (https://knative.dev/docs/serving/autoscaling/autoscaling-targets/).  # noqa: E501

        :param scale_target: The scale_target of this V1beta1ComponentExtensionSpec.  # noqa: E501
        :type: int
        """

        self._scale_target = scale_target

    @property
    def timeout(self):
        """Gets the timeout of this V1beta1ComponentExtensionSpec.  # noqa: E501

        TimeoutSeconds specifies the number of seconds to wait before timing out a request to the component.  # noqa: E501

        :return: The timeout of this V1beta1ComponentExtensionSpec.  # noqa: E501
        :rtype: int
        """
        return self._timeout

    @timeout.setter
    def timeout(self, timeout):
        """Sets the timeout of this V1beta1ComponentExtensionSpec.

        TimeoutSeconds specifies the number of seconds to wait before timing out a request to the component.  # noqa: E501

        :param timeout: The timeout of this V1beta1ComponentExtensionSpec.  # noqa: E501
        :type: int
        """

        self._timeout = timeout

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, V1beta1ComponentExtensionSpec):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, V1beta1ComponentExtensionSpec):
            return True

        return self.to_dict() != other.to_dict()



# Copyright 2023 The KServe Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# coding: utf-8

"""
    KServe

    Python SDK for KServe  # noqa: E501

    The version of the OpenAPI document: v0.1
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from kserve.configuration import Configuration


class V1alpha1InferenceRouter(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'router_type': 'str',
        'steps': 'list[V1alpha1InferenceStep]'
    }

    attribute_map = {
        'router_type': 'routerType',
        'steps': 'steps'
    }

    def __init__(self, router_type='', steps=None, local_vars_configuration=None):  # noqa: E501
        """V1alpha1InferenceRouter - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._router_type = None
        self._steps = None
        self.discriminator = None

        self.router_type = router_type
        if steps is not None:
            self.steps = steps

    @property
    def router_type(self):
        """Gets the router_type of this V1alpha1InferenceRouter.  # noqa: E501

        RouterType  - `Sequence:` chain multiple inference steps with input/output from previous step  - `Splitter:` randomly routes to the target service according to the weight  - `Ensemble:` routes the request to multiple models and then merge the responses  - `Switch:` routes the request to one of the steps based on condition  # noqa: E501

        :return: The router_type of this V1alpha1InferenceRouter.  # noqa: E501
        :rtype: str
        """
        return self._router_type

    @router_type.setter
    def router_type(self, router_type):
        """Sets the router_type of this V1alpha1InferenceRouter.

        RouterType  - `Sequence:` chain multiple inference steps with input/output from previous step  - `Splitter:` randomly routes to the target service according to the weight  - `Ensemble:` routes the request to multiple models and then merge the responses  - `Switch:` routes the request to one of the steps based on condition  # noqa: E501

        :param router_type: The router_type of this V1alpha1InferenceRouter.  # noqa: E501
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and router_type is None:  # noqa: E501
            raise ValueError("Invalid value for `router_type`, must not be `None`")  # noqa: E501

        self._router_type = router_type

    @property
    def steps(self):
        """Gets the steps of this V1alpha1InferenceRouter.  # noqa: E501

        Steps defines destinations for the current router node  # noqa: E501

        :return: The steps of this V1alpha1InferenceRouter.  # noqa: E501
        :rtype: list[V1alpha1InferenceStep]
        """
        return self._steps

    @steps.setter
    def steps(self, steps):
        """Sets the steps of this V1alpha1InferenceRouter.

        Steps defines destinations for the current router node  # noqa: E501

        :param steps: The steps of this V1alpha1InferenceRouter.  # noqa: E501
        :type: list[V1alpha1InferenceStep]
        """

        self._steps = steps

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, V1alpha1InferenceRouter):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, V1alpha1InferenceRouter):
            return True

        return self.to_dict() != other.to_dict()



# Copyright 2022 The KServe Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from typing import Optional, Union, Dict, List, AsyncIterator

from fastapi import Request, Response, FastAPI, APIRouter
from starlette.responses import StreamingResponse
from fastapi.responses import ORJSONResponse

from kserve.errors import ModelNotReady
from ..dataplane import DataPlane
from ..model_repository_extension import ModelRepositoryExtension
from ...constants.constants import V1_ROUTE_PREFIX


class V1Endpoints:
    """KServe V1 Endpoints"""

    def __init__(
        self,
        dataplane: DataPlane,
        model_repository_extension: Optional[ModelRepositoryExtension] = None,
    ):
        self.model_repository_extension = model_repository_extension
        self.dataplane = dataplane

    async def models(self) -> Dict[str, List[str]]:
        """Get a list of models in the model registry.

        Returns:
            Dict[str, List[str]]: List of model names.
        """
        return {"models": list(self.dataplane.model_registry.get_models().keys())}

    async def model_ready(self, model_name: str) -> Dict[str, Union[str, bool]]:
        """Check if a given model is ready.

        Args:
            model_name (str): Model name.

        Returns:
            Dict[str, Union[str, bool]]: Name of the model and whether it's ready.
        """
        model_ready = await self.dataplane.model_ready(model_name)
        if not model_ready:
            raise ModelNotReady(model_name)

        return {"name": model_name, "ready": model_ready}

    async def predict(self, model_name: str, request: Request) -> Union[Response, Dict]:
        """Predict request handler.

        It sends the request to the dataplane where the model will process the request body.

        Args:
            model_name (str): Model name.
            request (Request): Raw request object.

        Returns:
            Dict|Response: Model inference response.
        """
        model_ready = await self.dataplane.model_ready(model_name)

        if not model_ready:
            raise ModelNotReady(model_name)

        body = await request.body()
        headers = dict(request.headers.items())
        infer_request, req_attributes = self.dataplane.decode(
            body=body, headers=headers
        )
        response, response_headers = await self.dataplane.infer(
            model_name=model_name, request=infer_request, headers=headers
        )
        response, res_headers = self.dataplane.encode(
            model_name=model_name,
            response=response,
            headers=headers,
            req_attributes=req_attributes,
        )
        response_headers.update(res_headers)
        response_headers.pop("content-length", None)

        if isinstance(response, bytes) or isinstance(response, str):
            return Response(content=response, headers=response_headers)
        if isinstance(response, AsyncIterator):
            return StreamingResponse(content=response)
        return ORJSONResponse(content=response, headers=response_headers)

    async def explain(self, model_name: str, request: Request) -> Union[Response, Dict]:
        """Explain handler.

        Args:
            model_name (str): Model name.
            request (Request): Raw request object.

        Returns:
            Dict: Explainer output.
        """
        model_ready = await self.dataplane.model_ready(model_name)

        if not model_ready:
            raise ModelNotReady(model_name)

        body = await request.body()
        headers = dict(request.headers.items())
        infer_request, req_attributes = self.dataplane.decode(
            body=body, headers=headers
        )
        response, response_headers = await self.dataplane.explain(
            model_name=model_name, request=infer_request, headers=headers
        )
        response, res_headers = self.dataplane.encode(
            model_name=model_name,
            response=response,
            headers=response_headers,
            req_attributes=req_attributes,
        )

        response_headers.update(res_headers)
        response_headers.pop("content-length", None)

        if isinstance(response, dict):
            return ORJSONResponse(content=response, headers=response_headers)
        return Response(content=response, headers=response_headers)


def register_v1_endpoints(
    app: FastAPI,
    dataplane: DataPlane,
    model_repository_extension: Optional[ModelRepositoryExtension],
):
    """Register V1 endpoints.

    Args:
        app (FastAPI): FastAPI app.
        dataplane (DataPlane): DataPlane object.
        model_repository_extension (Optional[ModelRepositoryExtension]): Model repository extension.
    """
    v1_endpoints = V1Endpoints(dataplane, model_repository_extension)
    v1_router = APIRouter(prefix=V1_ROUTE_PREFIX, tags=["V1"])
    v1_router.add_api_route(r"/models", v1_endpoints.models, methods=["GET"])
    # Model Health API returns 200 if model is ready to serve.
    v1_router.add_api_route(
        r"/models/{model_name}", v1_endpoints.model_ready, methods=["GET"]
    )
    # Note: Set response_model to None to resolve fastapi Response issue
    # https://fastapi.tiangolo.com/tutorial/response-model/#disable-response-model
    v1_router.add_api_route(
        r"/models/{model_name}:predict",
        v1_endpoints.predict,
        response_model=None,
        methods=["POST"],
    )
    v1_router.add_api_route(
        r"/models/{model_name}:explain",
        v1_endpoints.explain,
        response_model=None,
        methods=["POST"],
    )
    app.include_router(v1_router)



# Copyright 2024 The KServe Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from typing import Optional
from ..inference_client import InferenceGRPCClient, InferenceRESTClient, RESTConfig
from .utils import is_v2


class InferenceClientFactory:
    _instance = None
    _grpc_client: Optional[InferenceGRPCClient] = None
    _rest_v1_client: Optional[InferenceRESTClient] = None
    _rest_v2_client: Optional[InferenceRESTClient] = None

    def __new__(cls):
        if cls._instance is None:
            cls._instance = super(InferenceClientFactory, cls).__new__(cls)
        return cls._instance

    def get_grpc_client(self, url: str, **kwargs) -> InferenceGRPCClient:
        if self._grpc_client is None:
            self._grpc_client = InferenceGRPCClient(url, **kwargs)
        return self._grpc_client

    def get_rest_client(self, config: RESTConfig = None) -> InferenceRESTClient:
        if config and is_v2(config.protocol):
            if self._rest_v2_client is None:
                self._rest_v2_client = InferenceRESTClient(config)
            return self._rest_v2_client
        if self._rest_v1_client is None:
            self._rest_v1_client = InferenceRESTClient(config)
        return self._rest_v1_client

    async def close(self):
        if self._grpc_client is not None:
            await self._grpc_client.close()
        if self._rest_v1_client is not None:
            await self._rest_v1_client.close()
        if self._rest_v2_client is not None:
            await self._rest_v2_client.close()



# Copyright 2023 The KServe Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# coding: utf-8

"""
    KServe

    Python SDK for KServe  # noqa: E501

    The version of the OpenAPI document: v0.1
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from kserve.configuration import Configuration


class V1alpha1BuiltInAdapter(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'env': 'list[V1EnvVar]',
        'mem_buffer_bytes': 'int',
        'model_loading_timeout_millis': 'int',
        'runtime_management_port': 'int',
        'server_type': 'str'
    }

    attribute_map = {
        'env': 'env',
        'mem_buffer_bytes': 'memBufferBytes',
        'model_loading_timeout_millis': 'modelLoadingTimeoutMillis',
        'runtime_management_port': 'runtimeManagementPort',
        'server_type': 'serverType'
    }

    def __init__(self, env=None, mem_buffer_bytes=None, model_loading_timeout_millis=None, runtime_management_port=None, server_type=None, local_vars_configuration=None):  # noqa: E501
        """V1alpha1BuiltInAdapter - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._env = None
        self._mem_buffer_bytes = None
        self._model_loading_timeout_millis = None
        self._runtime_management_port = None
        self._server_type = None
        self.discriminator = None

        if env is not None:
            self.env = env
        if mem_buffer_bytes is not None:
            self.mem_buffer_bytes = mem_buffer_bytes
        if model_loading_timeout_millis is not None:
            self.model_loading_timeout_millis = model_loading_timeout_millis
        if runtime_management_port is not None:
            self.runtime_management_port = runtime_management_port
        if server_type is not None:
            self.server_type = server_type

    @property
    def env(self):
        """Gets the env of this V1alpha1BuiltInAdapter.  # noqa: E501

        Environment variables used to control other aspects of the built-in adapter's behaviour (uncommon)  # noqa: E501

        :return: The env of this V1alpha1BuiltInAdapter.  # noqa: E501
        :rtype: list[V1EnvVar]
        """
        return self._env

    @env.setter
    def env(self, env):
        """Sets the env of this V1alpha1BuiltInAdapter.

        Environment variables used to control other aspects of the built-in adapter's behaviour (uncommon)  # noqa: E501

        :param env: The env of this V1alpha1BuiltInAdapter.  # noqa: E501
        :type: list[V1EnvVar]
        """

        self._env = env

    @property
    def mem_buffer_bytes(self):
        """Gets the mem_buffer_bytes of this V1alpha1BuiltInAdapter.  # noqa: E501

        Fixed memory overhead to subtract from runtime container's memory allocation to determine model capacity  # noqa: E501

        :return: The mem_buffer_bytes of this V1alpha1BuiltInAdapter.  # noqa: E501
        :rtype: int
        """
        return self._mem_buffer_bytes

    @mem_buffer_bytes.setter
    def mem_buffer_bytes(self, mem_buffer_bytes):
        """Sets the mem_buffer_bytes of this V1alpha1BuiltInAdapter.

        Fixed memory overhead to subtract from runtime container's memory allocation to determine model capacity  # noqa: E501

        :param mem_buffer_bytes: The mem_buffer_bytes of this V1alpha1BuiltInAdapter.  # noqa: E501
        :type: int
        """

        self._mem_buffer_bytes = mem_buffer_bytes

    @property
    def model_loading_timeout_millis(self):
        """Gets the model_loading_timeout_millis of this V1alpha1BuiltInAdapter.  # noqa: E501

        Timeout for model loading operations in milliseconds  # noqa: E501

        :return: The model_loading_timeout_millis of this V1alpha1BuiltInAdapter.  # noqa: E501
        :rtype: int
        """
        return self._model_loading_timeout_millis

    @model_loading_timeout_millis.setter
    def model_loading_timeout_millis(self, model_loading_timeout_millis):
        """Sets the model_loading_timeout_millis of this V1alpha1BuiltInAdapter.

        Timeout for model loading operations in milliseconds  # noqa: E501

        :param model_loading_timeout_millis: The model_loading_timeout_millis of this V1alpha1BuiltInAdapter.  # noqa: E501
        :type: int
        """

        self._model_loading_timeout_millis = model_loading_timeout_millis

    @property
    def runtime_management_port(self):
        """Gets the runtime_management_port of this V1alpha1BuiltInAdapter.  # noqa: E501

        Port which the runtime server listens for model management requests  # noqa: E501

        :return: The runtime_management_port of this V1alpha1BuiltInAdapter.  # noqa: E501
        :rtype: int
        """
        return self._runtime_management_port

    @runtime_management_port.setter
    def runtime_management_port(self, runtime_management_port):
        """Sets the runtime_management_port of this V1alpha1BuiltInAdapter.

        Port which the runtime server listens for model management requests  # noqa: E501

        :param runtime_management_port: The runtime_management_port of this V1alpha1BuiltInAdapter.  # noqa: E501
        :type: int
        """

        self._runtime_management_port = runtime_management_port

    @property
    def server_type(self):
        """Gets the server_type of this V1alpha1BuiltInAdapter.  # noqa: E501

        ServerType must be one of the supported built-in types such as \"triton\" or \"mlserver\", and the runtime's container must have the same name  # noqa: E501

        :return: The server_type of this V1alpha1BuiltInAdapter.  # noqa: E501
        :rtype: str
        """
        return self._server_type

    @server_type.setter
    def server_type(self, server_type):
        """Sets the server_type of this V1alpha1BuiltInAdapter.

        ServerType must be one of the supported built-in types such as \"triton\" or \"mlserver\", and the runtime's container must have the same name  # noqa: E501

        :param server_type: The server_type of this V1alpha1BuiltInAdapter.  # noqa: E501
        :type: str
        """

        self._server_type = server_type

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, V1alpha1BuiltInAdapter):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, V1alpha1BuiltInAdapter):
            return True

        return self.to_dict() != other.to_dict()



# Copyright 2022 The KServe Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from prometheus_client import Histogram
from pydantic import BaseModel

PROM_LABELS = ["model_name"]
PRE_HIST_TIME = Histogram(
    "request_preprocess_seconds", "pre-process request latency", PROM_LABELS
)
POST_HIST_TIME = Histogram(
    "request_postprocess_seconds", "post-process request latency", PROM_LABELS
)
PREDICT_HIST_TIME = Histogram(
    "request_predict_seconds", "predict request latency", PROM_LABELS
)
EXPLAIN_HIST_TIME = Histogram(
    "request_explain_seconds", "explain request latency", PROM_LABELS
)


class LLMStats(BaseModel):
    """LLM metrics data class."""

    num_prompt_tokens: int = 0
    num_generation_tokens: int = 0


def get_labels(model_name):
    return {PROM_LABELS[0]: model_name}
